###############################
#### Code to generate Fig1 ####
###############################

#### Load required packages
import networkx as nx
import pystan
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from datetime import datetime
from scipy import stats
import random
from copy import deepcopy
from networkx.readwrite import json_graph
import pystan
from os import path
import arviz
import pickle
import sys
import cmocean
import matplotlib.gridspec as gridspec
from matplotlib import rc
import matplotlib as mpl
from matplotlib.colorbar import Colorbar
from webweb import Web

cm_1 = cmocean.cm.amp
cm_2 = cmocean.cm.ice_r
cm_3 = cmocean.cm.tempo
cm_4 = cmocean.cm.rain
cm_5 = cmocean.cm.gray_r

#### Create Rc dataframe
col_names = ["simu_id", "case_id", "Rc"]
array_Rc = []
for i in range(100):
    G_chains = nx.read_gpickle("../data/sampled_chains/G_chains_"+str(i)+".gpickle")
    for node, attr in G_chains.nodes(data = True):
        #if not (attr["clinic_type"] == "1" and attr["diagtype"] != "1"):
        simu_id = i
        Rc = int(attr["R0"]) if attr["R0"] != "na" else np.nan
        attr_list = [simu_id, node, Rc]
        array_Rc.append(deepcopy(attr_list))
df_Rc = pd.DataFrame(data = array_Rc, columns = col_names)

#### Fit negative binomial distribution
for simu_id in range(100):
    if not path.isfile("../data/Rc_fit_nbinom/simu_"+str(simu_id)+".pkl"):
        Rc_nbinom_code = """
        data {
        int<lower=0> N; // number of patients
        int val[N]; 
        }

        parameters {
        real mu;
        real phi;
        }

        model {
        target += neg_binomial_2_lpmf(val|mu, phi);
        }

        """
        list_Rc =df_Rc.loc[df_Rc["simu_id"]==simu_id]["Rc"]
        Rc_data = {"N": len(list_Rc),
                "val": list_Rc
                }
        sm_nbinom = pystan.StanModel(model_code = Rc_nbinom_code)
        fit_Rc_nbinom = sm_nbinom.sampling(data = Rc_data, iter = 10000, chains = 1)
        list_of_draws_nbinom = fit_Rc_nbinom.extract()
        with open("../data/Rc_fit_nbinom/simu_"+str(simu_id)+".pkl", "wb") as wf:
            pickle.dump(list_of_draws_nbinom, wf, protocol=-1)

#### Fit geometric distribution
for simu_id in range(100):
    if not path.isfile("../data/Rc_fit_geom/simu_"+str(simu_id)+".pkl"):
        Rc_geom_code = """
        data {
            int<lower=0> L; // number of patients
            int n[L]; 
        }

        parameters {
            real<lower = 0, upper = 1> p;
        }

        model {
            for (i in 1:L) {
                target += n[i]*log(1-p) + log(p);
            }
        }

        """
        list_Rc =df_Rc.loc[df_Rc["simu_id"]==simu_id]["Rc"]
        Rc_data = {
            "L": len(list_Rc),
            "n": list_Rc
                }
        sm_geom = pystan.StanModel(model_code = Rc_geom_code)
        fit_Rc_geom = sm_geom.sampling(data = Rc_data, iter = 10000, chains = 1)
        list_of_draws_geom = fit_Rc_geom.extract()
        with open("../data/Rc_fit_geom/simu_"+str(simu_id)+".pkl", "wb") as wf:
            pickle.dump(list_of_draws_geom, wf, protocol=-1)

#### Fit Poisson distribution
for simu_id in range(100):
    if not path.isfile("../data/Rc_fit_poisson/simu_"+str(simu_id)+".pkl"):
        Rc_poisson_code = """
        data {
        int<lower=0> N; // number of patients
        int val[N]; 
        }

        parameters {
        real lambda;
        }

        model {
        target += poisson_lpmf(val|lambda);
        }

        """
        list_Rc =df_Rc.loc[df_Rc["simu_id"]==simu_id]["Rc"]
        Rc_data = {
            "N": len(list_Rc),
            "val": list_Rc
                }
        sm_poisson = pystan.StanModel(model_code = Rc_poisson_code)
        fit_Rc_poisson = sm_poisson.sampling(data = Rc_data, iter = 10000, chains = 1)
        list_of_draws_poisson = fit_Rc_poisson.extract()
        with open("../data/Rc_fit_poisson/simu_"+str(simu_id)+".pkl", "wb") as wf:
            pickle.dump(list_of_draws_poisson, wf, protocol=-1)


list_Rc = df_Rc["Rc"]
Rc_min = min(list_Rc)
Rc_max = max(list_Rc)

Rc_bins = np.arange(Rc_min, Rc_max+2, 1)
Rc_vals = Rc_bins[:-1]
Rc_PMFs_data = []
Rc_PMFs_nbinom = []
Rc_PMFs_geom = []
Rc_PMFs_poisson = []

list_nbinom_AIC = []
list_geom_AIC = []
list_poisson_AIC = []

list_var = []
list_mu = []
list_phi = []
for simu_id in range(100):
    Rc_PMF_data, _ = np.histogram(
        df_Rc[df_Rc["simu_id"]==simu_id]["Rc"],
        bins = Rc_bins,
        density=True
    )
    Rc_PMFs_data.append(Rc_PMF_data)
    list_of_draws_nbinom = pickle.load(open("../data/Rc_fit_nbinom/simu_"+str(simu_id)+".pkl", "rb"))
    
    for mu, phi in zip(list_of_draws_nbinom["mu"][1000:], list_of_draws_nbinom["phi"][1000:]):
        p = mu/(mu+phi)
        list_mu.append(mu)
        list_phi.append(phi)
        list_var.append(mu+mu**2/phi)
        Rc_PMFs_nbinom.append(stats.nbinom.pmf(Rc_vals, phi, 1-p))
        loglik = np.sum(stats.nbinom.logpmf(df_Rc[df_Rc["simu_id"]==simu_id]["Rc"], phi, 1-p))
        AIC = 2 * 2 - 2 * loglik
        list_nbinom_AIC.append(AIC)

    list_of_draws_geom = pickle.load(open("../data/Rc_fit_geom/simu_"+str(simu_id)+".pkl", "rb"))
    for p in list_of_draws_geom["p"][-100:]:
        Rc_PMFs_geom.append((1-p)**Rc_vals*p)
        loglik = np.sum(df_Rc[df_Rc["simu_id"]==simu_id]["Rc"]*np.log(1-p)+np.log(p))
        AIC = 2 * 1 - 2 * loglik
        list_geom_AIC.append(AIC)

    list_of_draws_poisson = pickle.load(open("../data/Rc_fit_poisson/simu_"+str(simu_id)+".pkl", "rb"))
    for lam in list_of_draws_poisson["lambda"][-100:]:
        Rc_PMFs_poisson.append(stats.poisson.pmf(Rc_vals, lam))
        loglik = np.sum(stats.poisson.logpmf(df_Rc[df_Rc["simu_id"]==simu_id]["Rc"], lam))
        AIC = 2 * 1 - 2 * loglik
        list_poisson_AIC.append(AIC)

#### Plot Rc raw and negative binomial fit
fig, axes0 = plt.subplots(figsize=(10, 4))
Rc_PMFs_data = np.array(Rc_PMFs_data)
Rc_PMF_data_mean = np.mean(Rc_PMFs_data, axis=0)
Rc_PMF_data_lower = np.percentile(Rc_PMFs_data,  2.5, axis=0)
Rc_PMF_data_upper = np.percentile(Rc_PMFs_data, 97.5, axis=0)
axes0.bar(Rc_vals - 0.2, Rc_PMF_data_mean, facecolor=cm_2(0.4), edgecolor=cm_2(0.7), label = "Data", width=0.35)
axes0.vlines(Rc_vals - 0.2, Rc_PMF_data_lower, Rc_PMF_data_upper, zorder=1000, color=cm_2(0.7), linewidth=1)
axes0.hlines(Rc_PMF_data_lower, Rc_vals - 0.1, Rc_vals - 0.3, color=cm_2(0.7), linewidth=1)
axes0.hlines(Rc_PMF_data_upper, Rc_vals - 0.1, Rc_vals - 0.3, color=cm_2(0.7), linewidth=1)

Rc_PMFs_nbinom = np.array(Rc_PMFs_nbinom)
Rc_PMF_nbinom_mean = np.mean(Rc_PMFs_nbinom, axis=0)
Rc_PMF_nbinom_lower = np.percentile(Rc_PMFs_nbinom,  2.5, axis=0)
Rc_PMF_nbinom_upper = np.percentile(Rc_PMFs_nbinom, 97.5, axis=0)
axes0.bar(Rc_vals + 0.2, Rc_PMF_nbinom_mean, facecolor=cm_1(0.4), edgecolor=cm_1(0.7), label="Negative binomial", width=0.35)
axes0.vlines(Rc_vals + 0.2, Rc_PMF_nbinom_lower, Rc_PMF_nbinom_upper, zorder=1000, color=cm_1(0.7), linewidth=1)
axes0.hlines(Rc_PMF_nbinom_lower, Rc_vals + 0.1, Rc_vals + 0.3, color=cm_1(0.7), linewidth=1)
axes0.hlines(Rc_PMF_nbinom_upper, Rc_vals + 0.1, Rc_vals + 0.3, color=cm_1(0.7), linewidth=1)

axes0.legend(frameon=False, fontsize=10, ncol=1, bbox_to_anchor=(1, 0.), loc="lower left")
axes0.set_xticks(Rc_vals)
axes0.set_ylim(-0.02, 0.81)
axes0.set_ylabel("Probability", fontsize=10)
axes0.set_xlabel("Number of secondary infections", fontsize=10)
axes0.spines['top'].set_visible(False)
axes0.spines['right'].set_visible(False)
axes0.set_xlim([-0.6, 10.7])

print("Negative binomial AIC: ", round(np.mean(list_nbinom_AIC)), "(", round(np.percentile(list_nbinom_AIC, 2.5)), round(np.percentile(list_nbinom_AIC, 97.5)), ")")
print("Geometric AIC: ", round(np.mean(list_geom_AIC)), "(", round(np.percentile(list_geom_AIC, 2.5)), round(np.percentile(list_geom_AIC, 97.5)), ")")
print("Poisson AIC: ", round(np.mean(list_poisson_AIC)), "(", round(np.percentile(list_poisson_AIC, 2.5)), round(np.percentile(list_poisson_AIC, 97.5)), ")")

plt.tight_layout()
plt.savefig("../plots/Fig1.pdf", format="pdf")

#### Visualize one realization of transmission chains
from webweb import Web
G_chains = nx.read_gpickle("../data/sampled_chains/G_chains_"+str(1)+".gpickle")
web = Web(nx_G = G_chains)
web.display.height = 900
web.display.width = 900
web.display.charge = 70
web.display.gravity = .5
web.display.colorBy = 'loc_viz'
web.display.colorPalette = 'Paired'
web.display.sizeBy = "degree"
web.show()