###############################
#### Code to generate Fig2 ####
###############################

#### Load required packages
import pandas as pd
import networkx as nx
import numpy as np
import math
from webweb import Web
import matplotlib.pyplot as plt
from datetime import datetime
from scipy import stats
from scipy import integrate
import random
from copy import deepcopy
from networkx.readwrite import json_graph
import pystan
import arviz
import matplotlib.ticker as ticker
from numba import cfunc
import scipy
import pickle
from os import path
import json
import cmocean
import matplotlib.gridspec as gridspec
from matplotlib import rc
import matplotlib as mpl
from matplotlib.colorbar import Colorbar

cm_1 = cmocean.cm.amp
cm_2 = cmocean.cm.ice_r
cm_3 = cmocean.cm.tempo
cm_4 = cmocean.cm.rain

cm1 = plt.get_cmap("Reds")
cm2 = plt.get_cmap("Blues")
cm3 = plt.get_cmap("Greens")
cm4 = plt.get_cmap("Oranges")


#### Create figure
fig = plt.figure(figsize=(11, 8))
gs = gridspec.GridSpec(10, 12, figure=fig)
ax00 = fig.add_subplot(gs[:3, :4])
ax01 = fig.add_subplot(gs[:3, 4:8])
ax02 = fig.add_subplot(gs[:3, 8:12])

ax10 = fig.add_subplot(gs[3:6, :6])
ax11 = fig.add_subplot(gs[3:6, 6:12])

ax20 = fig.add_subplot(gs[6:, :4])
ax21 = fig.add_subplot(gs[6:, 4:8])
ax22 = fig.add_subplot(gs[6:, 8:])


#### Load data
with open("../data/list_params_fit_infection_to_iso.json", "rb") as rf:
    list_params_fit_infection_to_iso = pickle.load(rf)
with open("../data/list_params_fit_onset_to_iso.json", "rb") as rf:
    list_params_fit_onset_to_iso = pickle.load(rf)
with open("../data/list_frac_pre_by_iso.json", "rb") as rf:
    list_frac_pre = pickle.load(rf)
with open("../data/list_trans_pre_by_iso.json", "rb") as rf:
    list_trans_pre = pickle.load(rf)
with open("../data/list_SIs_by_iso.json", "rb") as rf:
    list_SIs = pickle.load(rf)
with open("../data/list_GIs_by_iso.json", "rb") as rf:
    list_GIs = pickle.load(rf)
with open("../data/list_GI_adj.npy", "rb") as rf:
    list_GI_adj = np.load(rf)
with open("../data/list_onset_to_trans_adj.npy", "rb") as rf:
    list_onset_to_trans_adj = np.load(rf)

#### Plot GIs by time to isolation
y_ticks = [1, 2, 3, 4]
y_ticklabels = ["<2", "2-4", "4-6", ">6"]
violin_parts = ax00.violinplot(list_GIs, vert=False, widths=0.5)
for vp in violin_parts['bodies']:
    vp.set_facecolor(cm_1(0.4))
for partname in ('cbars','cmins','cmaxes'):
    vp = violin_parts[partname]
    vp.set_edgecolor(cm_1(0.9))
    vp.set_linewidth(0.5)
GI_mean = [np.mean(GIs) for GIs in list_GIs]
ax00.scatter(GI_mean, y_ticks, color=cm_1(0.9))
ax00.set_xlim(-2, 45)

ax00.set_yticks(y_ticks)
ax00.set_yticklabels(y_ticklabels)
ax00.tick_params(axis="both", labelsize=8)
ax00.set_xlabel(r"generation intervals (days)", fontsize=9)
ax00.set_ylabel("onset to isolation (days)", fontsize=9)
ax00.set_title("(A)", fontsize=10, loc="left")
ax00.set_facecolor("0.97")
ax00.set_ylim([0.5, 4.5])
ax00.spines['right'].set_visible(False)
ax00.spines['top'].set_visible(False)

#### Plot SIs by time to isolation
violin_parts = ax01.violinplot(list_SIs, vert=False, widths=0.5)
for vp in violin_parts['bodies']:
    vp.set_facecolor(cm_3(0.4))
for partname in ('cbars','cmins','cmaxes'):
    vp = violin_parts[partname]
    vp.set_edgecolor(cm_3(0.9))
    vp.set_linewidth(0.5)
SI_mean = [np.mean(SIs) for SIs in list_SIs]
ax01.scatter(SI_mean, y_ticks, color=cm_3(0.9))
ax01.set_xlim(-25, 35)
y_ticks = [1, 2, 3, 4]
y_ticklabels = ["<2", "2-4", "4-6", ">6"]
ax01.set_yticks(y_ticks)
ax01.set_yticklabels(y_ticklabels)
ax01.tick_params(axis="both", labelsize=8)
ax01.set_xlabel(r"serial intervals (days)", fontsize=9)
ax01.set_title("(B)", fontsize=10, loc="left")
ax01.set_facecolor("0.97")
ax01.set_ylim([0.5, 4.5])
ax01.spines['right'].set_visible(False)
ax01.spines['top'].set_visible(False)

#### Pot fraction of presymptomatic transmissioon by time to isolation
y_ticks = [1, 2, 3, 4]
y_ticklabels = ["<2", "2-4", "4-6", ">6"]
violin_parts = ax02.violinplot(list_frac_pre, vert=False)
for vp in violin_parts['bodies']:
    vp.set_facecolor(cm2(0.4))

for partname in ('cbars','cmins','cmaxes'):
    vp = violin_parts[partname]
    vp.set_edgecolor(cm_2(0.9))
    vp.set_linewidth(0.5)

ax02.scatter([np.mean(frac) for frac in list_frac_pre], y_ticks, color=cm_2(0.9), zorder=10)
ax02.set_yticks(y_ticks)
ax02.set_yticklabels(y_ticklabels)
ax02.set_xlabel("contribution of presymptomatic transmission", fontsize=9)

ax02.vlines(x=[0.5], ymin=0.5, ymax=4.5, color=cm_2(0.9), linestyle="-", linewidth=0.5)
ax02.set_xticks([i*0.1 for i in range(11)])
ax02.set_xticklabels([str(i*10)+"%" for i in range(11)], fontsize=8)
ax02.set_xlim(0.25, 0.98)
ax02.tick_params(axis="both", labelsize=8)
ax02.set_ylim([0.5, 4.5])
ax02.set_facecolor("0.97")
ax02.spines['right'].set_visible(False)
ax02.spines['top'].set_visible(False)
ax02.set_title("(C)", fontsize=10, loc="left")

#### Plot infectiousness profile since infection
growth_rate = 0.15
growth_rate_lower = 0.06
growth_rate_upper = 0.1


shape, loc, scale = stats.weibull_min.fit(list_GI_adj, floc=0)
def R_inverse(x, r, c, scale):
    ### generation interval
    GI = c/scale * (x/scale)**(c-1)*np.exp(-(x/scale)**c)
    return np.exp(-r*x) * GI
    
R_inv, _ = integrate.quad(R_inverse, 0, np.inf, args=(growth_rate, shape, scale))
R0_Wuhan = 1/R_inv
R_inv_lower, _ = integrate.quad(R_inverse, 0, np.inf, args=(growth_rate_lower, shape, scale))
R0_Wuhan_lower = 1/R_inv_lower
R_inv_upper, _ = integrate.quad(R_inverse, 0, np.inf, args=(growth_rate_upper, shape, scale))
R0_Wuhan_upper = 1/R_inv_upper


x = [0.01*i for i in range(2500)]

shape, loc, scale = stats.weibull_min.fit(list_GI_adj, floc=0)
y_Wuhan = stats.weibull_min.pdf(x, shape, loc, scale) * R0_Wuhan


#### Get exposure to isolation
shape = list_params_fit_infection_to_iso[0]["shape"]
loc = list_params_fit_infection_to_iso[0]["loc"]
scale = list_params_fit_infection_to_iso[0]["scale"]
y_filter_phase0 = 1. - stats.lognorm.cdf(x, shape, loc, scale)
y_intervention_phase0 = y_filter_phase0*y_Wuhan
trans_reduction_phase0 = sum(y_intervention_phase0)/sum(y_Wuhan)

shape = list_params_fit_infection_to_iso[1]["shape"]
loc = list_params_fit_infection_to_iso[1]["loc"]
scale = list_params_fit_infection_to_iso[1]["scale"]
y_filter_phase1 = 1. - stats.lognorm.cdf(x, shape, loc, scale)
y_intervention_phase1 = y_filter_phase1*y_Wuhan
trans_reduction_phase1 = sum(y_intervention_phase1)/sum(y_Wuhan)

shape = list_params_fit_infection_to_iso[2]["shape"]
loc = list_params_fit_infection_to_iso[2]["loc"]
scale = list_params_fit_infection_to_iso[2]["scale"]
y_filter_phase2 = 1. - stats.lognorm.cdf(x, shape, loc, scale)
y_intervention_phase2 = y_filter_phase2*y_Wuhan
trans_reduction_phase2= sum(y_intervention_phase2)/sum(y_Wuhan)


R0_str = str(round(R0_Wuhan, 2))# + " (" + str(round(R0_Wuhan_lower, 2)).zfill(2) + ", " + str(round(R0_Wuhan_upper, 2)) + ")"
ax10.plot(
    x, y_Wuhan,
    color=cm_1(0.7),
    linewidth=1,
    zorder=1,
    label="Wuhan before lockdown: "+r"$R_0$"+"="+R0_str
)
ax10.fill_between(
    x,
    y_Wuhan,
    y_intervention_phase0,
    color=cm_1(0.3), 
    alpha=0.5
)

R0_str_reduced_phase0 = str(round(R0_Wuhan*trans_reduction_phase0, 2))# + " (" + str(round(R0_Wuhan_lower*trans_reduction_phase0, 2)) + ", " + str(round(R0_Wuhan_upper*trans_reduction_phase0, 2)) + ")"
ax10.plot(
    x, y_intervention_phase0,
    color=cm_4(0.7),
    linewidth=1,
    zorder=2,
    linestyle="--",
    label="Phase   I: "+r"$R_0^E$"+"="+R0_str_reduced_phase0
)

ax10.fill_between(
    x,
    y_intervention_phase0,
    y_intervention_phase1,
    zorder=2,
    color=cm_4(0.3),
    alpha=0.5,
)

R0_str_reduced_phase1 = str(format(R0_Wuhan*trans_reduction_phase1, ".2f"))# + " (" + str(round(R0_Wuhan_lower*trans_reduction_phase1, 2)) + ", " + str(round(R0_Wuhan_upper*trans_reduction_phase1, 2)) + ")"
ax10.plot(
    x, y_intervention_phase1,
    color=cm_2(0.7),
    linewidth=1,
    zorder=3,
    linestyle="-.",
    label="Phase  II: "+r"$R_0^E$"+"="+R0_str_reduced_phase1
)

ax10.fill_between(
    x,
    y_intervention_phase1,
    y_intervention_phase2,
    zorder=3,
    color=cm2(0.5),
    alpha=0.5
)


R0_str_reduced_phase2 = str(format(R0_Wuhan*trans_reduction_phase2, ".2f"))# + " (" + str(round(R0_Wuhan_lower*trans_reduction_phase2, 2)) + ", " + str(round(R0_Wuhan_upper*trans_reduction_phase2, 2)) + ")"
ax10.plot(
    x, y_intervention_phase2,
    color=cm_3(0.7),
    linewidth=1,
    zorder=4,
    linestyle=":",
    label="Phase III: "+r"$R_0^E$"+"="+R0_str_reduced_phase2
)

ax10.fill_between(
    x,
    y_intervention_phase2,
    zorder=4,
    alpha=0.5,
    color=cm_3(0.3)
)


ax10.legend(loc="upper right", frameon=False, fontsize=8)
ax10.set_ylabel("infectiousness", fontsize=9)
ax10.set_xlabel("time since infection (days)", fontsize=9)
ax10.set_xlim([-1, 30])
ax10.set_ylim([0, 0.3])
ax10.set_yticks([0.05*i for i in range(7)])
ax10.spines['right'].set_visible(False)
ax10.spines['top'].set_visible(False)
ax10.tick_params(axis="both", labelsize=8)
ax10.set_title("(D)", loc="left", fontsize=10)
ax10.set_facecolor("0.97")


#### Plot infectiousness profile since onset
growth_rate = 0.15
growth_rate_lower = 0.14
growth_rate_upper = 0.17

shape, loc, scale = stats.weibull_min.fit(list_GI_adj, floc=0)
def R_inverse(x, r, c, scale):
    ### generation interval
    GI = c/scale * (x/scale)**(c-1)*np.exp(-(x/scale)**c)
    return np.exp(-r*x) * GI
    
R_inv, _ = integrate.quad(R_inverse, 0, np.inf, args=(growth_rate, shape, scale))
R0_Wuhan = 1/R_inv
R_inv_lower, _ = integrate.quad(R_inverse, 0, np.inf, args=(growth_rate_lower, shape, scale))
R0_Wuhan_lower = 1/R_inv_lower
R_inv_upper, _ = integrate.quad(R_inverse, 0, np.inf, args=(growth_rate_upper, shape, scale))
R0_Wuhan_upper = 1/R_inv_upper

x = [0.01*i for i in range(-1500, 1500)]

loc, scale = stats.norm.fit(list_onset_to_trans_adj)
y_Wuhan = stats.norm.pdf(x, loc, scale) * R0_Wuhan

shape = list_params_fit_onset_to_iso[0]["shape"]
loc = list_params_fit_onset_to_iso[0]["loc"]
scale = list_params_fit_onset_to_iso[0]["scale"]
y_filter_phase0 = 1. - stats.lognorm.cdf(x, shape, loc=loc, scale=scale)
y_intervention_phase0 = y_filter_phase0*y_Wuhan
trans_reduction_phase0 = sum(y_intervention_phase0)/sum(y_Wuhan)

loc = list_params_fit_onset_to_iso[1]["loc"]
scale = list_params_fit_onset_to_iso[1]["scale"]
y_filter_phase1 = 1. - stats.norm.cdf(x, loc=loc, scale=scale)
y_intervention_phase1 = y_filter_phase1*y_Wuhan
trans_reduction_phase1 = sum(y_intervention_phase1)/sum(y_Wuhan)

loc = list_params_fit_onset_to_iso[2]["loc"]
scale = list_params_fit_onset_to_iso[2]["scale"]
y_filter_phase2 = 1. - stats.laplace.cdf(x, loc=loc, scale=scale)
y_intervention_phase2 = y_filter_phase2*y_Wuhan
trans_reduction_phase2= sum(y_intervention_phase2)/sum(y_Wuhan)

R0_str = str(round(R0_Wuhan, 2)) + " (" + str(round(R0_Wuhan_lower, 2)) + ", " + str(round(R0_Wuhan_upper, 2)) + ")"
ax11.plot(
    x, y_Wuhan,
    color=cm_1(0.7),
    linewidth=1,
    zorder=2,
    label="Wuhan before lockdown"
)
ax11.fill_between(
    x,
    y_Wuhan,
    y_intervention_phase0,
    edgecolor=cm_1(0.3, 0.01),
    facecolor=cm_1(0.3),
    alpha=0.5,
    zorder=2
)

R0_str_reduced_phase0 = str(round(R0_Wuhan*trans_reduction_phase0, 2)) + " (" + str(round(R0_Wuhan_lower*trans_reduction_phase0, 2)) + ", " + str(round(R0_Wuhan_upper*trans_reduction_phase0, 2)) + ")"

ax11.plot(
    x, y_intervention_phase0,
    color=cm_4(0.7),
    linewidth=1,
    linestyle="--",
    zorder=3,
    label="Phase   I"
)

ax11.fill_between(
    x,
    y_intervention_phase0,
    y_intervention_phase1,
    zorder=3,
    edgecolor=cm_4(0.3, 0.01),
    facecolor=cm_4(0.3),
    alpha=0.5,
)

R0_str_reduced_phase1 = str(round(R0_Wuhan*trans_reduction_phase1, 2)) + " (" + str(round(R0_Wuhan_lower*trans_reduction_phase1, 2)) + ", " + str(round(R0_Wuhan_upper*trans_reduction_phase1, 2)) + ")"

ax11.plot(
    x, y_intervention_phase1,
    color=cm_2(0.7),
    linewidth=1,
    linestyle="-.",
    zorder=4,
    label="Phase  II"
)

ax11.fill_between(
    x,
    y_intervention_phase1,
    y_intervention_phase2,
    zorder=4,
    edgecolor=cm2(0.5, 0.01),
    facecolor=cm2(0.5),
    alpha=0.5
)


R0_str_reduced_phase2 = str(round(R0_Wuhan*trans_reduction_phase2, 2)) + " (" + str(round(R0_Wuhan_lower*trans_reduction_phase2, 2)) + ", " + str(round(R0_Wuhan_upper*trans_reduction_phase2, 2)) + ")"

ax11.plot(
    x, y_intervention_phase2,
    color=cm_3(0.7),
    linewidth=1,
    linestyle=":",
    zorder=5,
    label="Phase III"
)


ax11.fill_between(
    x,
    y_intervention_phase2,
    zorder=5,
    alpha=0.5,
    edgecolor=cm_3(0.3, 0.01),
    facecolor=cm_3(0.3)
)

ax11.vlines(0, 0, 0.3, linewidth=0.5, linestyle="-")
ax11.legend(loc="upper right", frameon=False, fontsize=8)
ax11.set_xlabel("time since onset (days)", fontsize=9)
ax11.set_xlim([-15, 15])
ax11.fill_between(
    [-15,0],
    [0, 0],
    [0.3, 0.3],
    zorder=1,
    edgecolor="0.8",
    facecolor="0.8",
    alpha=1,
    
)
ax11.text(-14.5, 0.03, "Pre-symptomatic", fontsize=8, ha="left")
ax11.text(14.5, 0.03, "Symptomatic", fontsize=8, ha="right")
ax11.set_ylim([0, 0.3])
ax11.set_yticks([0.05*i for i in range(7)])
ax11.spines['right'].set_visible(False)
ax11.spines['top'].set_visible(False)
ax11.tick_params(axis="both", labelsize=8)
ax11.set_title("(E)", loc="left", fontsize=10)
ax11.set_facecolor("0.97")


####  Plot intervention phase diagram 1
#### Get R0 in Wuhan
growth_rate = 0.08
shape, loc, scale = stats.weibull_min.fit(list_GI_adj, floc=0)
def R_inverse(x, r, c, scale):
    ### generation interval
    GI = c/scale * (x/scale)**(c-1)*np.exp(-(x/scale)**c)
    return np.exp(-r*x) * GI
R_inv, _ = integrate.quad(R_inverse, 0, np.inf, args=(growth_rate, shape, scale))
R0_Wuhan = 1/R_inv

#### Get list R0s
with open("../data/list_R0s_avg_iso.json", "rb") as rf:
    list_R0s, list_avg_iso = pickle.load(rf)

list_detection_rate = np.linspace(0.6, 1, 51)
array_R0s = np.zeros((51, 51))
for c0, detection_rate  in enumerate(list_detection_rate):
    for c1, R0 in enumerate(list_R0s):
        array_R0s[c0][c1] = 1 - (R0*detection_rate + R0_Wuhan*(1-detection_rate))/R0_Wuhan
        
cax0 = ax20.contourf(list_avg_iso, list_detection_rate, array_R0s, levels=200, cmap=cmocean.cm.balance, vmin=0.1, vmax=.7)
cbar0 = fig.colorbar(cax0, ax=ax20, ticks = [i*0.05 for i in range(21)], orientation="vertical", shrink=0.7, pad=0.03)
cbar0.ax.set_title(r"$\Delta {R_0}$", fontsize=8, loc="left")
cbar0.ax.tick_params(axis="both", labelsize=5, length=1)
cbar0.outline.set_visible(False)

x = list_avg_iso
y0 = [(R0_Wuhan - 0.5* R0_Wuhan)/(R0_Wuhan - R0) for R0 in list_R0s]
y1 = [(R0_Wuhan - 0.6* R0_Wuhan)/(R0_Wuhan - R0) for R0 in list_R0s]
y2 = [(R0_Wuhan - 0.7* R0_Wuhan)/(R0_Wuhan - R0) for R0 in list_R0s]
ax20.plot(x, y0, linestyle="--", color="0.4", linewidth=1)
ax20.plot(x, y1, linestyle="--", color="0.4", linewidth=1)
ax20.plot(x, y2, linestyle="--", color="0.4", linewidth=1)

ax20.text(x[10], y1[33], "50%", rotation=45, fontsize=8, color="0.3")
ax20.text(x[21], y1[24], "40%", rotation=45, fontsize=8, color="0.3")
ax20.text(x[33], y1[10], "30%", rotation=45, fontsize=8, color="0.3")

ax20.set_ylim(0.6, 1)
ax20.set_xticks([-2, -1, 0, 1, 2])
ax20.set_yticklabels([str(int((0.6+0.05*i)*100))+"%" for i in range(9)])
ax20.set_xlabel(r"$\tau_{iso}$ (days)", fontsize=9)
ax20.set_ylabel("% of infection isolated", fontsize=9)
ax20.set_title("(F)", loc="left", fontsize=10)
ax20.set_title(r"Reduction in $R_0$ (%)", loc="right", fontsize=10)
ax20.spines['right'].set_visible(False)
ax20.spines['top'].set_visible(False)
ax20.tick_params(axis="both", labelsize=8)

####  Plot intervention phase diagram 2
list_detection_rate = np.linspace(0.1, 0.8, 51)
list_trans_reduction = np.linspace(0.1, 0.35, 51)
array_R0s = np.zeros((51, 51))

for c0, detection_rate  in enumerate(list_detection_rate):
    for c1, trans_reduction in enumerate(list_trans_reduction):
        array_R0s[c0][c1] = (list_R0s[50]*detection_rate + R0_Wuhan*(1-detection_rate))*(1-trans_reduction)
        
x_list = list_trans_reduction
y_list = [(1/(1-x)-R0_Wuhan)/(list_R0s[50]-R0_Wuhan) for x in x_list]

cax1 = ax22.contourf(list_trans_reduction, list_detection_rate, array_R0s, levels=200, cmap=cmocean.cm.balance, vmin=0.5, vmax=1.5)
cbar1 = fig.colorbar(cax1, ax=ax22, ticks = [0.6 + i*0.1 for i in range(14)], orientation="vertical", shrink=0.7, pad=0.03)
cbar1.ax.set_title(r"$R_0^E$", fontsize=8, loc="left")
cbar1.ax.tick_params(axis="both", labelsize=5, length=1)
cbar1.outline.set_visible(False)

ax22.plot(x_list, y_list, linestyle="--", color="0.3", linewidth=1)
ax22.set_xticks([0.05*i + 0.1 for i in range(6)])
ax22.set_xticklabels([str(int((0.05*i+0.1)*100))+"%" for i in range(6)])
ax22.set_yticks([0.1*i for i in range(1, 9)])
ax22.set_yticklabels([str(int((0.1*i)*100))+"%" for i in range(1, 9)])
ax22.set_xlabel("% reduction in contact rate", fontsize=9)
ax22.set_ylabel("% of infection isolated", fontsize=9)
ax22.set_title("(H)", loc="left", fontsize=10)
ax22.spines['right'].set_visible(False)
ax22.spines['top'].set_visible(False)
ax22.set_ylim((0.1, 0.8))
ax22.tick_params(axis="both", labelsize=8)
ax22.set_title("$R_0$="+str(format(R0_Wuhan, ".2f"))+r", $\tau_{iso}$=2 (days)", loc="right", fontsize=10)

####  Plot intervention phase diagram 3
#### Get R0 in Wuhan
growth_rate = 0.15
shape, loc, scale = stats.weibull_min.fit(list_GI_adj, floc=0)
def R_inverse(x, r, c, scale):
    ### generation interval
    GI = c/scale * (x/scale)**(c-1)*np.exp(-(x/scale)**c)
    return np.exp(-r*x) * GI
R_inv, _ = integrate.quad(R_inverse, 0, np.inf, args=(growth_rate, shape, scale))
R0_Wuhan = 1/R_inv

#### Get list R0s
with open("../data/list_R0s_upper_avg_iso.json", "rb") as rf:
    list_R0s, list_avg_iso = pickle.load(rf)
list_detection_rate = np.linspace(0.2, 0.8, 51)
list_trans_reduction = np.linspace(0.25, 0.5, 51)
array_R0s = np.zeros((51, 51))

for c0, detection_rate  in enumerate(list_detection_rate):
    for c1, trans_reduction in enumerate(list_trans_reduction):
        array_R0s[c0][c1] = (list_R0s[50]*detection_rate + R0_Wuhan*(1-detection_rate))*(1-trans_reduction)
        
x_list = list_trans_reduction
y_list = [(1/(1-x)-R0_Wuhan)/(list_R0s[50]-R0_Wuhan) for x in x_list]

cax1 = ax21.contourf(list_trans_reduction, list_detection_rate, array_R0s, levels=200, cmap=cmocean.cm.balance, vmin=0.4, vmax=1.6)
cbar1 = fig.colorbar(cax1, ax=ax21, ticks = [0.5 + i*0.1 for i in range(21)], orientation="vertical", shrink=0.7, pad=0.03)
cbar1.ax.tick_params(axis="both", labelsize=6, length=1)
cbar1.ax.set_title(r"$R_0^E$", fontsize=8, loc="left")
cbar1.outline.set_visible(False)
ax21.plot(x_list, y_list, linestyle="--", color="0.3", linewidth=1)
ax21.set_xticks([0.05*i + 0.25 for i in range(6)])
ax21.set_xticklabels([str(int((0.05*i+0.25)*100))+"%" for i in range(6)])
ax21.set_yticks([0.1*i for i in range(2, 9)])
ax21.set_yticklabels([str(int((0.1*i)*100))+"%" for i in range(2, 9)])
ax21.set_xlabel("% reduction in contact rate", fontsize=9)
ax21.set_ylabel("% of infection isolated", fontsize=9)
ax21.set_title("(G)", loc="left", fontsize=10)
ax21.set_title("$R_0$="+str(format(R0_Wuhan, ".2f"))+r", $\tau_{iso}$=0 (days)", loc="right", fontsize=10)
ax21.spines['right'].set_visible(False)
ax21.spines['top'].set_visible(False)
ax21.set_ylim((0.2, 0.8))
ax21.tick_params(axis="both", labelsize=8)

plt.tight_layout(w_pad=1)
plt.savefig("../plots/Fig2.pdf")