###############################
#### Code to generate Fig3 ####
###############################

#### Load required packages
import pandas as pd
import networkx as nx
import numpy as np
import math
from webweb import Web
import matplotlib.pyplot as plt
from datetime import datetime
from scipy import stats
from scipy import integrate
import random
from copy import deepcopy
from networkx.readwrite import json_graph
import pystan
import arviz
import matplotlib.ticker as ticker
from numba import cfunc
import scipy
import pickle
from os import path
import os
import json
import cmocean
import time
import logging
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
import matplotlib.gridspec as gridspec
from matplotlib import rc
import matplotlib as mpl
import seaborn as sns

cm1 = plt.get_cmap("Reds")
cm2 = plt.get_cmap("Blues")
cm3 = plt.get_cmap("Greens")
cm4 = plt.get_cmap("Oranges")
cm5 = cmocean.cm.balance
colors = ["#D55740", "#6CB8D2", "#479E88", "#405283", "#E79F84", "#8691B1", "#9FCFC3", "#CA361E", "#7A624C", "#AD9D88"]

cm_1 = cmocean.cm.amp
cm_2 = cmocean.cm.ice_r
cm_3 = cmocean.cm.tempo
cm_4 = cmocean.cm.rain
cm_5 = cmocean.cm.gray_r
cm_6 = cmocean.cm.matter
list_cms = [cm_2, cm_3, cm_4, cm_1]

#### Load data
odds_ratios_community_contacts = pd.read_csv("../data/contact_pattern_odds_ratios_community.csv", header=0, index_col=0)
odds_ratios_family_contacts = pd.read_csv("../data/contact_pattern_odds_ratios_family.csv", header=0, index_col=0)
odds_ratios_social_contacts = pd.read_csv("../data/contact_pattern_odds_ratios_social.csv", header=0, index_col=0)
odds_ratios_household_contacts = pd.read_csv("../data/contact_pattern_odds_ratios_household.csv", header=0, index_col=0)
list_thetas = list(pd.read_csv("../data/contact_pattern_thetas.csv")["theta"])

with open("../data/list_risks_contact_type.json", "rb") as rf:
    list_risks = pickle.load(rf)
with open("../data/list_d_expos_contact_type.json", "rb") as rf:
    list_d_expos = pickle.load(rf)
with open("../data/list_degrees_contact_type.json", "rb") as rf:
    list_degrees = pickle.load(rf)
with open("../data/list_contact_matrices_contact_type.json", "rb") as rf:
    list_contact_matrices = pickle.load(rf)

#### Create figure instance
fig = plt.figure(figsize=(13, 12))
gs = gridspec.GridSpec(16, 4, figure=fig)

ax00 = fig.add_subplot(gs[:2, 0])
ax01 = fig.add_subplot(gs[:2, 1])
ax02 = fig.add_subplot(gs[:2, 2])
ax03 = fig.add_subplot(gs[:2, 3])
axes0 = [ax00, ax01, ax02, ax03]

ax10 = fig.add_subplot(gs[2:4, 0])
ax11 = fig.add_subplot(gs[2:4, 1])
ax12 = fig.add_subplot(gs[2:4, 2])
ax13 = fig.add_subplot(gs[2:4, 3])
axes1 = [ax10, ax11, ax12, ax13]

ax20 = fig.add_subplot(gs[4:6, 0])
ax21 = fig.add_subplot(gs[4:6, 1])
ax22 = fig.add_subplot(gs[4:6, 2])
ax23 = fig.add_subplot(gs[4:6, 3])
axes2 = [ax20, ax21, ax22, ax23]

ax30 = fig.add_subplot(gs[6:10, 0])
ax31 = fig.add_subplot(gs[6:10, 1])
ax32 = fig.add_subplot(gs[6:10, 2])
ax33 = fig.add_subplot(gs[6:10, 3])
axes3 = [ax30, ax31, ax32, ax33]

ax40 = fig.add_subplot(gs[10:16, 0])
ax41 = fig.add_subplot(gs[10:16, 1])
ax42 = fig.add_subplot(gs[10:16, 2])
ax43 = fig.add_subplot(gs[10:16, 3])
axes4 = [ax40, ax41, ax42, ax43]

def get_dispersion_k(var_list):
    return(np.mean(var_list)**2/(np.std(var_list)**2-np.mean(var_list)))

#### Plot risk
list_titles = ["Commmunity", "Social", "Extended Family", "Household"]
contact_types = [["Community contactsI", "Community contactsII+"], ["Social contactsI", "Social contactsII+"], ["Family contactsI", "Family contactsII+"], ["Household contactsI", "Household contactsII+"]]
for idx in range(4):
    cm = list_cms[idx]
    violin_parts = axes0[idx].violinplot([list_risks[idx]], [1], vert=False, widths=0.5)
    print(list_titles[idx]+"average with 95%CI: ", np.average(list_risks[idx])*100, np.percentile(list_risks[idx], 2.5)*100, np.percentile(list_risks[idx], 97.5)*100) 
    axes0[idx].set_xlabel("Transmission risk", fontsize=9)
    axes0[idx].set_title(list_titles[idx], fontsize=9, loc="left", pad=-5)
    for vp in violin_parts['bodies']:
        vp.set_facecolor(cm(0.3))
        vp.set_alpha(1)
    for partname in ('cbars','cmins','cmaxes'):
        vp = violin_parts[partname]
        vp.set_edgecolor(cm(0.9))
        vp.set_linewidth(1)
    axes0[idx].scatter([np.average(list_risks[idx])], [1], s=50, color=cm(0.8))
    axes0[idx].text(np.average(list_risks[idx])*1.1, 0.85, str(round(np.average(list_risks[idx])*100, 2))+"%", color=cm(0.8), fontsize=9)
    axes0[idx].set_ylim([0.6, 1.4])
    axes0[idx].set_yticks([])
    axes0[idx].tick_params(axis='both', which='both', labelsize=8)
    axes0[idx].spines["right"].set_visible(False)
    axes0[idx].spines["left"].set_visible(False)
    axes0[idx].spines["top"].set_visible(False)

#### Plot duration of exposures
list_cms = [cm_2, cm_3, cm_4, cm_1]
list_titles = ["Community", "Social", "Extended Family", "Household"]
contact_types = [["Community contactsI", "Community contactsII+"], ["Social contactsI", "Social contactsII+"], ["Family contactsI", "Family contactsII+"], ["Household contactsI", "Household contactsII+"]]
for idx in range(4):
    cm = list_cms[idx]
    list_d_expo = list_d_expos[idx]
    bins = np.linspace(0, 50, num=51)
    pdf, bins = np.histogram(list_d_expo, bins=bins, density=True)
    cdf = np.cumsum(pdf)
    ccdf = 1. - cdf
    # normalize by bin width

    # plot it!
    axes1[idx].plot(bins[:-1], ccdf, color=cm(0.5), linewidth=2)
    axes1[idx].vlines(np.average(list_d_expo), 0, 1,  color=cm(0.7), linewidth=1, linestyle="--")
    axes1[idx].text(np.average(list_d_expo)+0.5, 0.8, str(round(np.average(list_d_expo), 1)), horizontalalignment="left", color=cm(0.8), fontsize=10)
    axes1[idx].set_xlabel("Duration of exposure (days)", fontsize=9)
    if idx == 0:
        axes1[idx].set_ylabel("CCDF", fontsize=9)
    axes1[idx].spines['right'].set_visible(False)
    axes1[idx].spines['top'].set_visible(False)
    axes1[idx].tick_params(axis='both', which='both', labelsize=8)
    axes1[idx].set_title(list_titles[idx], fontsize=9, loc="left")


#### Plot degree
list_cms = [cm_2, cm_3, cm_4, cm_1]
list_titles = ["Community", "Social", "Extended Family", "Household"]
contact_types = ["community_degree", "social_degree", "family_degree", "household_degree"]
for idx in range(4):
    cm = list_cms[idx]
    list_degree = list_degrees[idx]
    bins = np.logspace(0, np.log10(200), num=13)
    widths = (bins[1:] - bins[:-1])


    hist = np.histogram(list_degree, bins=bins)
    # normalize by bin width
    hist_norm = hist[0]/widths

    # plot it!
    axes2[idx].bar(bins[:-1], hist_norm, width=widths*0.8, color=list_cms[idx](0.4), edgecolor=cm(0.8))
    axes2[idx].set_xscale("log")
    axes2[idx].set_yscale("log")
    axes2[idx].set_xlabel("Degree", fontsize=9)
    if idx == 0:
        axes2[idx].set_ylabel("PMF", fontsize=9)
    axes2[idx].spines['right'].set_visible(False)
    axes2[idx].spines['top'].set_visible(False)
    axes2[idx].tick_params(axis='both', which='both', labelsize=8)
    axes2[idx].set_title(list_titles[idx], fontsize=9, loc="left")
    axes2[idx].set_title("Dispersion: k="+str(round(get_dispersion_k(list_degree), 2)), fontsize=9, loc="right")
    axes2[idx].set_xlim([0.7, 110])
    axes2[idx].set_ylim([0.01, 1000])
    axes2[idx].vlines(np.average(list_degree), 0.01, 1000, color=cm(0.8), linestyle="--", linewidth=1)
    axes2[idx].text(np.average(list_degree)+0.5, 800, str(round(np.average(list_degree), 1)), horizontalalignment="left", verticalalignment="top", color=cm(0.8), fontsize=10)
        
#### Heat map age degree
list_cms = [cm_2, cm_3, cm_4, cm_1]
list_titles = ["Community", "Social", "Extended Family", "Household"]
contact_types = [["Community contactsI", "Community contactsII+"], ["Social contactsI", "Social contactsII+"], ["Family contactsI", "Family contactsII+"], ["Household contactsI", "Household contactsII+"]]
for idx in range(4):
    cm = list_cms[idx]
    sns.kdeplot(list_contact_matrices[idx][0], list_contact_matrices[idx][1],
                kernel = "gau",
                bw=0.2,
                cmap=cm,
                shade=True,
                shade_lowest=True,
                cbar=False,
                cbar_kws={"shrink": 0.5, "pad": 0.03, "anchor": (1,1)},
                n_levels = 15,
                ax=axes3[idx])
    if idx == 0:
        axes3[idx].set_ylabel("Age case", fontsize=9)
    else:
        axes3[idx].set_ylabel("", fontsize=9)
        axes3[idx].set_yticklabels([])
        axes3[idx].set_yticks([])
    axes3[idx].set_xlabel("Age contact", fontsize=9)
    axes3[idx].spines['right'].set_visible(False)
    axes3[idx].spines['top'].set_visible(False)
    axes3[idx].tick_params(axis='both', which='both', labelsize=8)
    axes3[idx].set_title(list_titles[idx], fontsize=9, loc="left", pad=-5)
    axes3[idx].set_xticks([0, 10, 20, 30, 40, 50, 60, 70, 80, 90])
    axes3[idx].set_xlim([0, 90])
    axes3[idx].set_ylim([0, 90])
    axes3[idx].set_facecolor(cm(1/30.))


#### Plot regression
list_odds_ratios = [odds_ratios_community_contacts, odds_ratios_social_contacts, odds_ratios_family_contacts, odds_ratios_household_contacts]
list_xlims = [[0.05, 20], [0.1, 10], [0.1, 10], [0.1, 10], [0.1, 10]]
list_xticks = [[0, 1, 2, 3, 4, 5], [0, 1, 2, 3], [0, 1, 2, 3], [0, 1, 2, 3], [0, 1, 2, 3]]

#list_titles = ["Community", "Social", "Family", "Household"]
for idx, temp_odds_ratios in enumerate(list_odds_ratios):
    cm = list_cms[idx]
    list_predictors = [
        "iso_to_onset",
        "pre_01_250",
        "na",
        "na",
        "travel_Wuhan1",
        "na",
        "symp_dry_cough1",
        "na",
        "symp_fever1",
        "na",
        "sexmale",
        "age_cat65+",
        "na",
        "age_cat<26",
    ]
    
    list_predictors_labels = [
        "Isolation to onset (days)",
        "After Jan. 25",
        "Before Jan. 25",
        "Travel history Wuhan (N)",
        "Travel history Wuhan (Y)",
        "Dry cough (N)",
        "Dry cough (Y)",
        "Fever (N)",
        "Fever (Y)",
        "Female",
        "Male",
        "65+ years",
        "26-64 years",
        "0-25 years",
    ]

    y = []
    x = []
    x_lower = []
    x_upper = []
    colors = []
    edgecolors = []

    list_text = []
    x_text =[]
    y_text = []
    text_colors = []

    for i, predictor in enumerate(list_predictors):
        y.append(i + 1)
        if predictor != "na":
            x.append(temp_odds_ratios.loc[predictor, "Estimate"])
            x_lower.append(temp_odds_ratios.loc[predictor, "lower"])
            x_upper.append(temp_odds_ratios.loc[predictor, "upper"])
            x_text.append(temp_odds_ratios.loc[predictor, "Estimate"])
            if temp_odds_ratios.loc[predictor, "Pr...z.."] < 0.001:
                significance = "***"
            elif temp_odds_ratios.loc[predictor, "Pr...z.."] < 0.01:
                significance = "**"
            elif temp_odds_ratios.loc[predictor, "Pr...z.."] < 0.05:
                significance = "*"
            elif temp_odds_ratios.loc[predictor, "Pr...z.."] < 0.1:
                significance = ""
            else:
                significance = ""
            list_text.append(str(round(temp_odds_ratios.loc[predictor, "Estimate"], 2))+significance)
            y_text.append(i + 1)
        else:
            x.append(1)
            x_lower.append(1)
            x_upper.append(1)
            x_text.append(1)
            list_text.append("  Ref.")
            y_text.append(i + 1)

        if predictor == "na":
            edgecolors.append("0")
            colors.append("1")
            text_colors.append("0.")
        elif predictor in ["iso_to_onset", "pre_01_250"]:
            colors.append(cm(0.7))
            edgecolors.append(cm(0.7))
            text_colors.append(cm(0.7))
        else:
            colors.append(cm(0.35))
            edgecolors.append(cm(0.35))
            text_colors.append(cm(0.35))

    if idx == 0:
        axes4[idx].set_yticks(y)
        axes4[idx].set_yticklabels(list_predictors_labels, fontsize=8, rotation=0)
    else:
        axes4[idx].set_yticks([])
    axes4[idx].scatter(x, y,  marker="o", c=colors, edgecolors=edgecolors, s=20, zorder=3)
    axes4[idx].hlines(y, x_lower, x_upper, color=edgecolors, linewidth=0.5, zorder=10)
    axes4[idx].vlines(1, 0.3, max(y)+0.8, linestyle="-", color="0", zorder=1, linewidth=0.5)
    axes4[idx].set_xlim(0.1, 10)
    axes4[idx].set_ylim([0.3, max(y)+0.8])
    
    axes4[idx].set_xlabel("Rate ratio", fontsize=8)

    axes4[idx].fill_between(list_xlims[idx], [.3, .3], [1.5, 1.5], color=cm(0.15), zorder=0)
    axes4[idx].fill_between(list_xlims[idx], [1.5, 1.5], [3.5, 3.5], color=cm(0.05), zorder=0)
    axes4[idx].fill_between(list_xlims[idx], [3.5, 3.5], [5.5, 5.5], color=cm(0.15), zorder=0)
    axes4[idx].fill_between(list_xlims[idx], [5.5, 5.5], [7.5, 7.5], color=cm(0.05), zorder=0)
    axes4[idx].fill_between(list_xlims[idx], [7.5, 7.5], [9.5, 9.5], color=cm(0.15), zorder=0)
    axes4[idx].fill_between(list_xlims[idx], [9.5, 9.5], [11.5, 11.5], color=cm(0.05), zorder=0)
    axes4[idx].fill_between(list_xlims[idx], [11.5, 11.5], [15.8, 15.8], color=cm(0.15), zorder=0)
    axes4[idx].set_title(list_titles[idx], loc="left", fontsize=9, pad=-5)
    
    y_text = np.array(y_text) - 0.2
    for x_t, y_t, text, color in zip(x_text, y_text, list_text, text_colors):
        axes4[idx].text(x_t, y_t, text, color=color, horizontalalignment="left", verticalalignment="top", fontsize=6)
    axes4[idx].spines['right'].set_visible(False)
    axes4[idx].spines['top'].set_visible(False)
    axes4[idx].tick_params(axis='both', which='both', labelsize=8)
    axes4[idx].set_xscale("log")
plt.figtext(0.09, 0.98, "(A)", fontsize=11)
plt.figtext(0.09, 0.86, "(B)", fontsize=11)
plt.figtext(0.09, 0.736, "(C)", fontsize=11)
plt.figtext(0.09, 0.615, "(D)", fontsize=11)
plt.figtext(0.09, 0.363, "(E)", fontsize=11)

plt.tight_layout(h_pad=1, w_pad=0)
plt.savefig("../plots/Fig3.pdf", format="pdf")