#################################
#### Code to generate FigS10 ####
#################################

#### Load required packages
import numpy as np
from scipy import stats
import random
import cmocean
import matplotlib.pyplot as plt
import matplotlib.gridspec as gridspec
from matplotlib import rc
import matplotlib as mpl
from matplotlib.colorbar import Colorbar
from copy import deepcopy

cm1 = cmocean.cm.amp
cm2 = cmocean.cm.ice_r
cm3 = cmocean.cm.tempo
cm4 = cmocean.cm.rain

#### Load data
array_GI_censored = np.load("../data/array_GI_censored.npy", allow_pickle=True)
array_onset_to_trans_censored = np.load("../data/array_onset_to_trans_censored.npy", allow_pickle=True)
array_incu_infector = np.load("../data/array_incubation_infector_censored.npy", allow_pickle=True)
array_incu_infectee = np.load("../data/array_incubation_infectee_censored.npy", allow_pickle=True)
array_p_escape_iso = np.load("../data/array_p_escape_iso.npy", allow_pickle=True)

array_p_incu_2d = []
array_weights_adj = []

for i in range(100):
    array_p_incu_2d.append([])
    for incu_infector, incu_infectee in zip(array_incu_infector[i], array_incu_infectee[i]):            
        p_incu_infector = stats.weibull_min.pdf(incu_infector, 1.578104 , scale=7.112123)
        p_incu_infectee = stats.weibull_min.pdf(incu_infectee, 1.578104 , scale=7.112123)
        array_p_incu_2d[i].append(p_incu_infector * p_incu_infectee)
    incu_kernel_imp = stats.gaussian_kde([array_incu_infector[i], array_incu_infectee[i]])
    p_kernel_2d_imp = incu_kernel_imp([array_incu_infector[i], array_incu_infectee[i]])
    array_weights_adj.append(np.array(array_p_incu_2d[i])/p_kernel_2d_imp/np.array(array_p_escape_iso[i]))

array_GI_adj = []
array_onset_to_trans_adj = []
list_GI_adj = []
list_onset_to_trans_adj = []
for i in range(100):
    array_GI_adj.append(random.choices(population=array_GI_censored[i], weights=array_weights_adj[i], k=len(array_GI_censored[i])))
    list_GI_adj += array_GI_adj[i]
    array_onset_to_trans_adj.append(random.choices(population=array_onset_to_trans_censored[i], weights=array_weights_adj[i], k=len(array_onset_to_trans_censored[i])))
    list_onset_to_trans_adj += array_onset_to_trans_adj[i]
np.save("../data/list_GI_adj.npy", list_GI_adj)
np.save("../data/list_onset_to_trans_adj.npy", list_onset_to_trans_adj)

list_onset_to_trans_adj = []
for row in array_onset_to_trans_adj:
    list_onset_to_trans_adj += row

cmaps = [cm1, cm2]
fig, axes = plt.subplots(2, 2, figsize = [8, 6])
params_GI_onset_to_trans_adj = []
for idx, temp_array in enumerate([array_GI_adj, array_onset_to_trans_adj]):
    temp_list = []
    for row in temp_array:
        temp_list += row
    lower = np.percentile(temp_list, 25)
    median = np.median(temp_list)
    upper = np.percentile(temp_list, 75)

    lower_str = str(round(lower, 1))
    median_str = str(round(median, 1))
    upper_str = str(round(upper, 1))

    if idx == 0:
        min_x = 0
        max_x = 25
    else:
        min_x = -15
        max_x = 10
    x_fit_CI = np.linspace(min_x, max_x, 10)
    x_fit = np.linspace(min_x, max_x, 200)

    array_y_fit_cdf = []
    for row in temp_array:
        row_sorted = np.sort(row)
        cdf = 1. * np.arange(len(row_sorted)) / (len(row_sorted) - 1)
        if idx == 0:
            shape, loc, scale = stats.weibull_min.fit(row, floc=0)
            array_y_fit_cdf.append(stats.weibull_min.cdf(x_fit_CI, shape, loc, scale=scale))
        else:
            shape, loc, scale = stats.lognorm.fit(row)
            array_y_fit_cdf.append(stats.lognorm.cdf(x_fit_CI, shape, loc, scale=scale))
        axes[0][idx].plot(row_sorted, cdf, linewidth=0.5, color=cm2(0.3))
    array_y_fit_cdf = np.array(array_y_fit_cdf)  

    if idx == 0:
        #### fit gamma distribution ####
        shape, loc, scale = stats.gamma.fit(temp_list, floc=0)
        y_fit_gamma = stats.gamma.pdf(x_fit, shape, loc, scale=scale)
        y_fit_gamma_cdf = stats.gamma.cdf(x_fit, shape, loc, scale=scale)
        AIC = 3 - int(stats.gamma.logpdf(temp_list, shape, loc, scale).sum())
        axes[0][idx].plot(
            x_fit, y_fit_gamma_cdf,
            linestyle = '--',
            linewidth=1.,
            color=cm2(0.7),
            label="gamma: AIC="+str(AIC))

        #### fit weibull distribution ####
        shape, loc, scale = stats.weibull_min.fit(temp_list, floc=0)
        y_fit_weibull_pdf = stats.weibull_min.pdf(x_fit, shape, loc, scale=scale)
        y_fit_weibull_cdf = stats.weibull_min.cdf(x_fit, shape, loc, scale=scale)
        y_fit_weibull_cdf_CI = stats.weibull_min.cdf(x_fit_CI, shape, loc, scale=scale)
        AIC = 2 - int(stats.weibull_min.logpdf(temp_list, shape, loc, scale).sum())
        params_str = "shape: "+str(round(shape, 2))+", "+"scale: "+str(round(scale, 2))
        axes[1][idx].plot(
            x_fit, y_fit_weibull_pdf,
            linestyle='-',
            color=cm1(0.7),
            linewidth=2,
            label="best fit")
        axes[0][idx].plot(
            x_fit, y_fit_weibull_cdf,
            linestyle='-',
            color=cm1(0.7),
            linewidth=2,
            label="weibull: AIC="+str(AIC))
        y_fit_cdf_lower = np.percentile(array_y_fit_cdf, 2.5, axis=0)
        y_fit_cdf_upper = np.percentile(array_y_fit_cdf, 97.5, axis=0)
        y_fit_cdf_avg = np.average(array_y_fit_cdf, axis=0)
        axes[0][idx].vlines(x_fit_CI, y_fit_cdf_lower, y_fit_cdf_upper, zorder=100, linewidth=1, color=cm1(0.7))
        
        #### fit lognormal distribution ####
        shape, loc, scale = stats.lognorm.fit(temp_list, floc=0)
        y_fit_lognormal = stats.lognorm.pdf(x_fit, shape, loc, scale=scale)
        y_fit_lognormal_cdf = stats.lognorm.cdf(x_fit, shape, loc, scale=scale)
        AIC = 3 - int(stats.lognorm.logpdf(temp_list, shape, loc, scale).sum())
        axes[0][idx].plot(
            x_fit, y_fit_lognormal_cdf,
            linestyle=":",
            linewidth=1.,
            color=cm3(0.7),
            label="lognormal: AIC="+str(AIC))
        
    else:
        #### fit gamma distribution ####
        shape, loc, scale = stats.gamma.fit(temp_list)
        y_fit_gamma = stats.gamma.pdf(x_fit, shape, loc, scale=scale)
        y_fit_gamma_cdf = stats.gamma.cdf(x_fit, shape, loc, scale=scale)
        AIC = 3 - int(stats.gamma.logpdf(temp_list, shape, loc, scale).sum())
        axes[0][idx].plot(
            x_fit, y_fit_gamma_cdf,
            linestyle=":",
            linewidth=1.,
            color=cm3(0.8),
            label="gamma: AIC="+str(AIC))
        
        #### fit normal distribution ####
        loc, scale = stats.norm.fit(temp_list)
        y_fit_norm = stats.norm.pdf(x_fit, loc, scale=scale)
        y_fit_norm_cdf = stats.norm.cdf(x_fit, loc, scale=scale)
        AIC = 2 - int(stats.norm.logpdf(temp_list, loc, scale).sum())
        axes[0][idx].plot(
            x_fit, y_fit_norm_cdf,
            linestyle="-",
            color=cm1(0.7),
            linewidth=2.,
            label="normal: AIC="+str(AIC))
        axes[1][idx].plot(
            x_fit, y_fit_norm,
            linestyle="-",
            linewidth=2,
            color=cm1(0.7),
            label="best fit")
        y_fit_cdf_lower = np.percentile(array_y_fit_cdf, 2.5, axis=0)
        y_fit_cdf_upper = np.percentile(array_y_fit_cdf, 97.5, axis=0)
        y_fit_cdf_avg = np.average(array_y_fit_cdf, axis=0)
        axes[0][idx].vlines(x_fit_CI, y_fit_cdf_lower, y_fit_cdf_upper, zorder=100, linewidth=1, color=cm1(0.7))
        
        #### fit lognormal distribution ####
        shape, loc, scale = stats.lognorm.fit(temp_list)
        y_fit_lognormal = stats.lognorm.pdf(x_fit, shape, loc, scale=scale)
        y_fit_lognormal_cdf = stats.lognorm.cdf(x_fit, shape, loc, scale=scale)
        y_fit_lognormal_cdf_CI = stats.lognorm.cdf(x_fit_CI, shape, loc, scale=scale)
        AIC = 3 - int(stats.lognorm.logpdf(temp_list, shape, loc, scale).sum())

        axes[0][idx].plot(
            x_fit, y_fit_lognormal_cdf,
            linestyle="--",
            linewidth=1,
            color=cm2(0.7),
            label="lognormal: AIC="+str(AIC))
        y_fit_cdf_lower = np.percentile(array_y_fit_cdf, 2.5, axis=0)
        y_fit_cdf_upper = np.percentile(array_y_fit_cdf, 97.5, axis=0)
        y_fit_cdf_avg = np.average(array_y_fit_cdf, axis=0)

    bar_width = (max(temp_list) - min(temp_list))/16
    axes[1][idx].hist(
        temp_list,
        bins=15,
        color="0.8",
        density=True,
        width=bar_width,
        label="data")

    if idx == 0:
        label_str = median_str + " ("  +lower_str + ", " + upper_str + ")"
        axes[0][idx].set_title("(A)", fontsize=10, loc="left")
        axes[1][idx].set_title("(C)", fontsize=10, loc="left")
        axes[0][idx].legend(fontsize=6, frameon=False, framealpha=None, loc="lower right", edgecolor="0.8", fancybox=False, borderaxespad=1)
        axes[1][idx].legend(fontsize=6, frameon=False, framealpha=None, loc="upper right", edgecolor="0.8", fancybox=False, borderaxespad=1)
        axes[0][idx].set_xlabel("Generation interval (days)", fontsize=8)
        axes[1][idx].set_xlabel("Generation interval (days)", fontsize=8)
    else:
        axes[0][idx].legend(fontsize=6, frameon=False, framealpha=None, loc="lower right", edgecolor="0.8", fancybox=False, borderaxespad=1)
        axes[1][idx].legend(fontsize=6, frameon=False, framealpha=None, loc="upper right", edgecolor="0.8", fancybox=False, borderaxespad=1)
        label_str = median_str + " ("  +lower_str + ", " + upper_str + ")" 
        axes[0][idx].set_title("(B)", fontsize=10, loc="left")
        axes[1][idx].set_title("(D)", fontsize=10, loc="left")
        axes[0][idx].set_xlabel("Onset to transmission (days)", fontsize=8)
        axes[1][idx].set_xlabel("Onset to transmission (days)", fontsize=8)
    
    axes[1][idx].set_ylabel("Probability density function", fontsize=8)
    axes[0][idx].set_ylabel("Cumulative distribution function", fontsize=8)
    axes[0][idx].tick_params(axis="both", labelsize=8)
    axes[1][idx].tick_params(axis="both", labelsize=8) 
    axes[1][idx].set_ylim(0, 0.15)
    
plt.tight_layout()
plt.savefig("../plots/FigS10.pdf")