################################
#### Code to generate FigS2 ####
################################

#### Load required packages
import pandas as pd
import numpy as np
import cmocean
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
import matplotlib.gridspec as gridspec
from matplotlib import rc
import matplotlib as mpl
import pickle
import networkx as nx
from webweb import Web

cm_1 = cmocean.cm.amp
cm_2 = cmocean.cm.ice_r
cm_3 = cmocean.cm.tempo
cm_4 = cmocean.cm.rain
cm_5 = cmocean.cm.gray_r
cm_6 = cmocean.cm.matter

#### Load data
with open("../data/dic_contact_statistics.json", "rb") as rf:
    dic_contact_statistics = pickle.load(rf)

#### Create figure instance and plot
fig, axes = plt.subplots(5, 1, figsize=(6, 10))

list_colors = [cm_1(0.4), cm_2(0.4), cm_3(0.4), cm_1(0.7), cm_5(0.4)]
list_colors_dark = [cm_1(0.7), cm_2(0.7), cm_3(0.7), cm_1(0.4), cm_5(0.7)]
list_loc = ["Changsha", "Yueyang", "Shaoyang", "Loudi", "Changde"]
list_age_cat = ["<13", "13-25", "26-64", "65+"]
list_contact_type = ["Household contacts", "Family contacts", "Social contacts", "Community contacts"]
list_contact_label = ["Household", "Extended family", "Social", "Community"]

for idx0, loc in enumerate(list_loc):
    for idx1, contact_type in enumerate(list_contact_type):
        avg_contacts = dic_contact_statistics[loc][contact_type]["avg_contacts"]
        x = []
        for id2, age_cat in enumerate(list_age_cat):
            x.append(id2 + 0.8 + 0.1*idx1)
        y = dic_contact_statistics[loc][contact_type]["age_distribution"]
        axes[idx0].bar(x, y, width=0.09, label=list_contact_label[idx1]+": "+str(avg_contacts)+" contacts per case", color=list_colors[idx1], zorder=0, align="edge")
    axes[idx0].set_ylim([0, 1])
    axes[idx0].set_xlabel("Age categories (years)")
    axes[idx0].set_ylabel("Probability", fontsize=10, rotation=-90, labelpad=15)
    axes[idx0].set_xticks([1, 2, 3, 4])
    axes[idx0].set_xticklabels(["<13", "13-25", "26-64", "65+"], fontsize=10)
    axes[idx0].yaxis.set_label_position("right")
    axes[idx0].yaxis.tick_right()
    axes[idx0].tick_params(axis='both', which='both', labelsize=10, labelleft=False, labelright=True)
    axes[idx0].text(0.95, 0.95, loc, fontsize=10, transform=axes[idx0].transAxes, horizontalalignment="right", verticalalignment="top")
    #axes[idx0].set_title("Age distribution (contacts)")
    axes[idx0].legend(frameon=False, fontsize=8, loc="upper left")
    axes[idx0].spines['left'].set_visible(False)
    axes[idx0].spines['top'].set_visible(False)
plt.tight_layout()
plt.savefig("../plots/FigS2.pdf")


#### Visualize transmission network
G_net = nx.DiGraph()
for i in range(0, 100):
    G_chains = nx.read_gpickle("../data/sampled_chains/G_chains_"+str(i)+".gpickle")
    for node in G_chains.nodes:
        if G_net.has_node(node):
            pass
        else:
            G_net.add_node(
                node,
                loc_viz=G_chains.nodes[node]["loc_viz"]
            )
    for p, s, w in G_chains.edges(data=True):
        if G_net.has_edge(p, s):
            G_net[p][s]["weight"] += 0.01
        else:
            G_net.add_edge(p, s, weight=0.01)

web = Web(nx_G = G_net)
web.display.height = 900
web.display.width = 900
web.display.charge = 70
web.display.gravity = .5
web.display.sizeBy = "strength"
web.display.colorBy = 'loc_viz'
web.display.colorPalette = 'Paired'
web.show()