################################
#### Code to generate FigS3 ####
################################

#### Load required packages
import pandas as pd
import numpy as np
import cmocean
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
import matplotlib.gridspec as gridspec
from matplotlib import rc
import matplotlib as mpl
import pickle
import seaborn as sns


cm_1 = cmocean.cm.amp
cm_2 = cmocean.cm.ice_r
cm_3 = cmocean.cm.tempo
cm_4 = cmocean.cm.rain
cm_5 = cmocean.cm.gray_r
cm_6 = cmocean.cm.matter

#### Create figure instance
fig = plt.figure(figsize=(12, 11))
gs = gridspec.GridSpec(10, 11, figure=fig)
ax0 = fig.add_subplot(gs[:, :6])
ax1 = fig.add_subplot(gs[:4, 6:])
ax2 = fig.add_subplot(gs[4:8, 6:])
ax3 = fig.add_subplot(gs[8:, 6:])

#### Load data
odds_ratios_imputed = pd.read_csv("../data/transmission_odds_ratios_imputed.csv", header=0, index_col=0)

#### Regression plot
dic_predictors_idx = {
    "intercept": "(Intercept)",

    "contact <13": "age_cat<13",
    "contact 13-25": "age_cat13-25",
    "contact 26-64": "na",
    "contact 65+": "age_cat65+",
    "contact female": "na",
    "contact male": "sexmale",

    "infector <26": "age_cat_source<26",
    "infector 26-64": "na",
    "infector 65+": "age_cat_source65+",
    "infector female": "na",
    "infector male": "sex_sourcemale",

    "asymptomatic": "severity_grouped_v1_source1",
    "mild & moderate": "na",
    "severe & critical": "severity_grouped_v1_source3",
    "fever 1": "symp_fever_source1",
    "fever 0": "na",
    "dry_cough 1": "symp_dry_cough_source1",
    "dry_cough 0": "na",

    "Wuhan 1": "source_Wuhan1",
    "Wuhan 0": "na",

    
    "household (pre)": "contact_type_updated_agg_v3Household contactsI",
    "ext. family (pre)": "na",
    "soical (pre)": "contact_type_updated_agg_v3Social contactsI",
    "community (pre)": "contact_type_updated_agg_v3Community contactsI",
    "healthcare (pre)": "contact_type_updated_agg_v3Healthcare contactsI",
    "household (post)": "contact_type_updated_agg_v3Household contactsII+",
    "ext. family (post)": "contact_type_updated_agg_v3Family contactsII+",
    "soical (post)": "contact_type_updated_agg_v3Social contactsII+",
    "community (post)": "contact_type_updated_agg_v3Community contactsII+",
    "healthcare (post)": "contact_type_updated_agg_v3Healthcare contactsII+",
    
    "duration of exposure": "duration_of_expo_cap",
    "onset within exposure 0": "na",
    "onset within exposure 1": "onset_within_expo1"
}

list_predictors = [
    "contact <13",
    "contact 13-25",
    "contact 26-64",
    "contact 65+",
    "contact male",
    "contact female",

    "infector <26",
    "infector 26-64",
    "infector 65+",
    "infector male",
    "infector female",
    "asymptomatic",
    "mild & moderate",
    "severe & critical",
    "fever 1",
    "fever 0",
    "dry_cough 1",
    "dry_cough 0",
    "Wuhan 1",
    "Wuhan 0",

    "household (pre)",
    "ext. family (pre)",   
    "soical (pre)",
    "community (pre)",
    "healthcare (pre)",
    "household (post)",
    "ext. family (post)",
    "soical (post)",
    "community (post)",
    "healthcare (post)",
    
    "duration of exposure",
    "onset within exposure 0",
    "onset within exposure 1"
]

list_predictors_labels = [
    "0-12 years",
    "13-25 years",
    "26-64 years",
    "65+ years",
    "Male",
    "Female",

    "0-25 years",
    "26-64 years",
    "65+ years",
    "Male",
    "Female",
    "Asymptomatic",
    "Mild & moderate",
    "Severe & critical",
    "Yes",
    "No",
    "Yes",
    "No",
    "Yes",
    "No",

    "Household",
    "Extended Family",
    "Social",
    "Community",
    "Healthcare",
    "Household",
    "Extended Family",
    "Social",
    "Community",
    "Healthcare",
    
    "Duration of Exposure (day)",
    "No",
    "Yes"
]

y = []
x = []
x_lower = []
x_upper = []
colors = []
edgecolors = []

list_text = []
list_text_full = []
x_text =[]
y_text = []
text_colors = []

for i, predictor in enumerate(list_predictors):
    y.append(len(list_predictors) - i)
    idx = dic_predictors_idx[predictor]
    if idx != "na":
        x.append(odds_ratios_imputed.loc[idx, "estimate"])
        x_lower.append(odds_ratios_imputed.loc[idx, "lower"])
        x_upper.append(odds_ratios_imputed.loc[idx, "upper"])
        x_text.append(odds_ratios_imputed.loc[idx, "estimate"])
        if odds_ratios_imputed.loc[idx, "p.value"] < 0.001:
            significance = "***"
        elif odds_ratios_imputed.loc[idx, "p.value"] < 0.01:
            significance = "**"
        elif odds_ratios_imputed.loc[idx, "p.value"] < 0.05:
            significance = "*"
        elif odds_ratios_imputed.loc[idx, "p.value"] < 0.1:
            significance = ""
        else:
            significance = ""
        list_text.append(str(round(odds_ratios_imputed.loc[idx, "estimate"], 2))+significance)
        list_text_full.append(str(round(odds_ratios_imputed.loc[idx, "estimate"], 2))+significance+" ("+str(round(odds_ratios_imputed.loc[idx, "lower"], 2))+", "+str(round(odds_ratios_imputed.loc[idx, "upper"], 2))+")")
        y_text.append(len(list_predictors) - i)
    else:
        x.append(1)
        x_lower.append(1)
        x_upper.append(1)
        x_text.append(1)
        list_text.append("  Ref.")
        list_text_full.append(" Ref.")
        y_text.append(len(list_predictors) - i)
        
    if x_upper[i] < 1:
        colors.append(cm_2(0.7))
        edgecolors.append(cm_2(0.7))
        text_colors.append(cm_2(0.7))
    elif x_lower[i] > 1:
        colors.append(cm_1(0.7))
        edgecolors.append(cm_1(0.7))
        text_colors.append(cm_1(0.7))
    elif idx == "na":
        edgecolors.append("0")
        colors.append("1")
        text_colors.append("0.")
    else:
        colors.append("0.7")
        edgecolors.append("0.7")
        text_colors.append("0.")
        
        

ax0.set_yticks(y)
ax0.set_yticklabels(list_predictors_labels, fontsize=10)
ax0.scatter(x, y,  marker="o", c=colors, edgecolors=edgecolors, s=30, zorder=3)
ax0.hlines(y, x_lower, x_upper, color=edgecolors, linewidth=0.5, zorder=10)
ax0.vlines(1, 0.3, max(y)+0.5, linestyle="-", color=cm_2(0.7), zorder=1, linewidth=0.5)
ax0.set_ylim([0.3, max(y)+0.5])

ax0.set_xlabel("Odds Ratio", fontsize=10)
ax0.fill_between([0.00, 100], [.3, .3], [2.4, 2.4], color=cm_2(0.05), zorder=0)
ax0.fill_between([0.00, 100], [2.4, 2.4], [3.4, 3.4], color=cm_2(0.15), zorder=0)
ax0.fill_between([0.00, 100], [3.4, 3.4], [13.4, 13.4], color=cm_2(0.05), zorder=0)
ax0.fill_between([0.00, 100], [13.4, 13.4], [15.4, 15.4], color=cm_2(0.15), zorder=0)
ax0.fill_between([0.00, 100], [15.4, 15.4], [17.4, 17.4], color=cm_2(0.05), zorder=0)
ax0.fill_between([0.00, 100], [17.4, 17.4], [19.4, 19.4], color=cm_2(0.15), zorder=0)
ax0.fill_between([0.00, 100], [19.4, 19.4], [22.4, 22.4], color=cm_2(0.05), zorder=0)
ax0.fill_between([0.00, 100], [22.4, 22.4], [24.4, 24.4], color=cm_2(0.15), zorder=0)
ax0.fill_between([0.00, 100], [24.4, 24.4], [27.4, 27.4], color=cm_2(0.05), zorder=0)
ax0.fill_between([0.00, 100], [27.4, 27.4], [29.4, 29.4], color=cm_2(0.15), zorder=0)
ax0.fill_between([0.00, 100], [29.4, 29.4], [33.4, 33.4], color=cm_2(0.05), zorder=0)

color=cm_2(0.8)

ax0.text(0.0035, 33.3, "Age (contact)\nsusceptibility", horizontalalignment="left", verticalalignment="top", color=color, fontsize=10)
ax0.text(0.0035, 29.3, "Sex (contact)\nsusceptibility", horizontalalignment="left", verticalalignment="top", color=color, fontsize=10)
ax0.text(0.0035, 27.3, "Age (case)\ninfectiousness", horizontalalignment="left", verticalalignment="top", color=color, fontsize=10)
ax0.text(0.0035, 24.3, "Sex (case)\ninfectiousness", horizontalalignment="left", verticalalignment="top", color=color, fontsize=10)
ax0.text(0.0035, 22.3, "Clinical severity (case)", horizontalalignment="left", verticalalignment="top", color=color, fontsize=10)
ax0.text(0.0035, 19.3, "\"Fever\" (case)", horizontalalignment="left", verticalalignment="top", color=color, fontsize=10)
ax0.text(0.0035, 17.3, "\"Dry cough\" (case)", horizontalalignment="left", verticalalignment="top", color=color, fontsize=10)
ax0.text(0.0035, 15.3, "Travel history Wuhan (case)", horizontalalignment="left", verticalalignment="top", color=color, fontsize=10)
ax0.text(0.0035, 13.3, "Contact settings", horizontalalignment="left", verticalalignment="top", color=color, fontsize=10, zorder=11)
ax0.text(9, 8.5, "Pre  Jan. 25, 2020", horizontalalignment="right", verticalalignment="bottom", color="0", fontsize=8, zorder=11)
ax0.text(0.0035, 8.3, "Post  Jan. 25, 2020", horizontalalignment="left", verticalalignment="top", color="0", fontsize=8, zorder=11)
ax0.text(0.0035, 3.3, "Duration of exposure", horizontalalignment="left", verticalalignment="top", color=color, fontsize=10)
ax0.text(0.0035, 2.3, "Onset within exposure", horizontalalignment="left", verticalalignment="top", color=color, fontsize=10)

ax0.hlines(8.4, 0.00, 10, linewidth=0.5, linestyle="--")

baseline_odds_str = str(round(odds_ratios_imputed.loc["(Intercept)", "estimate"], 2)) + " (" + str(round(odds_ratios_imputed.loc["(Intercept)", "lower"], 2)) + ", "+str(round(odds_ratios_imputed.loc["(Intercept)", "upper"], 2))+")"
ax0.set_title("Baseline Odds: " + baseline_odds_str, loc="right", fontsize=10)
ax0.set_title("(A)", fontsize=10, loc="left")
ax0.spines['right'].set_visible(False)
ax0.spines['top'].set_visible(False)
ax0.set_xlim([0.003, 10])
ax0.tick_params(axis='both', which='both', labelsize=10)
ax0.set_xscale("log")


y_text = np.array(y_text) - 0.2
for x_t, y_t, text, text_full, color in zip(x_text, y_text, list_text, list_text_full, text_colors):
    ax0.text(x_t, y_t, text, color=color, horizontalalignment="left", verticalalignment="top", fontsize=8)

sub_ax0 = ax0.inset_axes([0.09, 0.292, 0.4, 0.07])
df_mobility = pd.read_csv("../data_raw/Hunan_mobility.csv")
x =  [i for i in range(len(df_mobility["current"]))]
y1 = df_mobility["current"]
y2 = df_mobility["past"]

sub_ax0.plot(x, y1, color=cm_1(0.7), linewidth=.5, label="2020")
sub_ax0.plot(x, y2, color=cm_2(0.7), linewidth=.5, label="2019", linestyle=":")
sub_ax0.legend(loc="lower left", fontsize=7, frameon=False)
sub_ax0.set_ylim(1, 6)
sub_ax0.set_ylabel("mobility index", fontsize=7, labelpad=2)
xticks = [0, 31, 60]
xticklabels = ["Jan. 1st", "Feb. 1st", "Mar. 1st"]
sub_ax0.set_xticks(xticks)
sub_ax0.set_xticklabels(xticklabels)
sub_ax0.vlines(24, 0, 6, color="0.4", linewidth=.5, linestyle="--")
sub_ax0.tick_params(axis='both', which='major', labelsize=6)
sub_ax0.set_yticks([0, 1, 2, 3, 4, 5, 6])
sub_ax0.set_facecolor(cm_2(0.05))
sub_ax0.spines['top'].set_visible(False)
sub_ax0.spines['right'].set_visible(False)
sub_ax0.text(25, 5, "Jan. 25", fontsize=7, horizontalalignment="left")


#### Heatmap
with open("../data/age_pairs_infector_infectee_observation.json", "rb") as rf:
    age_pairs_observation = pickle.load(rf)
with open("../data/age_pairs_infector_infectee_projection.json", "rb") as rf:
    age_pairs_projection = pickle.load(rf)

sns.kdeplot(age_pairs_projection[0], age_pairs_projection[1],
            kernel = "gau",
            bw=0.2,
            cmap=cm_4,
            shade=True,
            shade_lowest=True,
            cbar=False,
            n_levels=20,
            thresh=0,
            ax=ax1)
ax1.set_title("Age Pattern (Projection)", fontsize=10, loc="right")
ax1.set_title("(B)", fontsize=10, loc="left")
ax1.set_xlabel("Age (contact)", fontsize=10)
ax1.set_ylabel("Age (case)", fontsize=10)
ax1.spines['right'].set_visible(False)
ax1.spines['top'].set_visible(False)
ax1.tick_params(axis='both', which='both', labelsize=10)
ax1.set_xticks([0, 10, 20, 30, 40, 50, 60, 70, 80, 90])
ax1.set_yticks([0, 10, 20, 30, 40, 50, 60, 70, 80, 90])
ax1.set_xlim([0, 90])
ax1.set_ylim([0, 90])
ax1.set_facecolor(cm_4(1/30.))


sns.kdeplot(age_pairs_observation[0], age_pairs_observation[1],
            kernel = "gau",
            bw=0.2,
            cmap=cm_1,
            shade=True,
            shade_lowest=True,
            cbar=False,
            n_levels=20,
            thresh=0,
            ax=ax2)

ax2.set_title("Age Pattern (Observation)", fontsize=10, loc="right")
ax2.set_title("(C)", fontsize=10, loc="left")
ax2.set_xlabel("Age (contact)", fontsize=10)
ax2.set_ylabel("Age (case)", fontsize=10)
ax2.spines['right'].set_visible(False)
ax2.spines['top'].set_visible(False)
ax2.tick_params(axis='both', which='both', labelsize=10)
ax2.set_xticks([0, 10, 20, 30, 40, 50, 60, 70, 80, 90])
ax2.set_yticks([0, 10, 20, 30, 40, 50, 60, 70, 80, 90])
ax2.set_xlim([0, 90])
ax2.set_ylim([0, 90])
ax2.set_facecolor(cm_1(1/30.))

with open("../data/list_relative_contribution_by_contact_types.json", "rb") as rf:
    list_relative_contribution_by_contact_types = pickle.load(rf)
with open("../data/relative_contribution_by_contact_types.json", "rb") as rf:
    relative_contribution_by_contact_types = pickle.load(rf)

#### transmission by setting
violin_parts = ax3.violinplot(
    list_relative_contribution_by_contact_types,
    widths=0.5,
    vert=True)

list_cms = [cm_2, cm_3, cm_4, cm_1]
facecolors = [cm_2(0.5), cm_3(0.5), cm_4(0.5), cm_1(0.5)]
edgecolors = [cm_2(0.8), cm_3(0.8), cm_4(0.8), cm_1(0.8)]

for partname in ('cbars','cmins','cmaxes'):
    vp = violin_parts[partname]
    vp.set_edgecolor(cm_2(0.9))
    vp.set_linewidth(0.5)

for idx, vp in enumerate(violin_parts['bodies']):
    vp.set_facecolor(cm_2(0.5))
ax3.scatter([1, 2, 3, 4, 5], relative_contribution_by_contact_types, s=30, color=cm_2(0.9), label="observations")
ax3.legend(frameon=False, fontsize=10)
ax3.set_xticks([1, 2, 3, 4, 5])
ax3.set_ylim([-0.05, 0.6])
ax3.set_xticklabels(["Household", "Ext. Family", "Social", "Community", "Healthcare"], fontsize=9, rotation=-0, horizontalalignment="center")
ax3.tick_params(axis='y', which='both', labelsize=10)
ax3.set_title("(D)", fontsize=10, loc="left")
ax3.set_ylabel("Fraction", fontsize=10)
ax3.set_facecolor("0.95")
ax3.spines['right'].set_visible(False)
ax3.spines['top'].set_visible(False)
    

plt.tight_layout()
plt.savefig('../plots/FigS3.pdf')