################################
#### Code to generate FigS4 ####
################################

#### Load required packages
import pandas as pd
import numpy as np
import cmocean
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
import matplotlib.gridspec as gridspec
from matplotlib import rc
import matplotlib as mpl


cm_1 = cmocean.cm.amp
cm_2 = cmocean.cm.ice_r
cm_3 = cmocean.cm.tempo
cm_4 = cmocean.cm.rain
cm_5 = cmocean.cm.gray_r
cm_6 = cmocean.cm.matter

#### Load data
odds_ratios_rna = pd.read_csv("../data/transmission_odds_ratios_rna.csv", header=0, index_col=0)

#### Create figure instance
fig, ax = plt.subplots(figsize=(7, 10))

#### Regression plot
dic_predictors_idx = {
    "intercept": "(Intercept)",

    "contact <13": "age_cat<13",
    "contact 13-25": "age_cat13-25",
    "contact 26-64": "na",
    "contact 65+": "age_cat65+",
    "contact female": "na",
    "contact male": "sexmale",

    "infector <13": "age_cat_source_finer<13",
    "infector 13-25": "age_cat_source_finer13-25",
    "infector 26-64": "na",
    "infector 65+": "age_cat_source_finer65+",
    "infector female": "na",
    "infector male": "sex_sourcemale",

    "asymptomatic": "severity_grouped_v1_expo_source1",
    "mild & moderate (-)": "na",
    "severe & critical (-)": "severity_grouped_v1_expo_source3-0",
    "mild & moderate (+)": "severity_grouped_v1_expo_source2-1",    
    "severe & critical (+)": "severity_grouped_v1_expo_source3-1",
    "fever 1": "symp_fever_source1",
    "fever 0": "na",
    "dry_cough 1": "symp_dry_cough_source1",
    "dry_cough 0": "na",

    "Wuhan 1": "source_Wuhan1",
    "Wuhan 0": "na",

    
    "household (pre)": "contact_type_updated_agg_v3Household contactsI",
    "ext. family (pre)": "na",
    "soical (pre)": "contact_type_updated_agg_v3Social contactsI",
    "community (pre)": "contact_type_updated_agg_v3Community contactsI",
    "healthcare (pre)": "contact_type_updated_agg_v3Healthcare contactsI",
    "household (post)": "contact_type_updated_agg_v3Household contactsII+",
    "ext. family (post)": "contact_type_updated_agg_v3Family contactsII+",
    "soical (post)": "contact_type_updated_agg_v3Social contactsII+",
    "community (post)": "contact_type_updated_agg_v3Community contactsII+",
    
    
    "duration of exposure": "duration_of_expo_cap",
}

list_predictors = [
    "contact <13",
    "contact 13-25",
    "contact 26-64",
    "contact 65+",
    "contact male",
    "contact female",

    "infector <13",
    "infector 13-25",
    "infector 26-64",
    "infector 65+",
    "infector male",
    "infector female",
    "asymptomatic",
    "mild & moderate (-)",
    "severe & critical (-)",
    "mild & moderate (+)",
    "severe & critical (+)",
    "fever 1",
    "fever 0",
    "dry_cough 1",
    "dry_cough 0",
    "Wuhan 1",
    "Wuhan 0",

    "household (pre)",
    "ext. family (pre)",   
    "soical (pre)",
    "community (pre)",
    "healthcare (pre)",
    "household (post)",
    "ext. family (post)",
    "soical (post)",
    "community (post)",
    
    
    "duration of exposure",
]

list_predictors_labels = [
    "0-12 years",
    "13-25 years",
    "26-64 years",
    "65+ years",
    "Male",
    "Female",

    "0-12 years",
    "13-25 years",
    "26-64 years",
    "65+ years",
    "Male",
    "Female",
    "Asymptomatic",
    "Mild & moderate (-)",
    "Severe & critical (-)",
    "Mild & moderate (+)",
    "Severe & critical (+)",
    "Yes",
    "No",
    "Yes",
    "No",
    "Yes",
    "No",

    "Household",
    "Extended Family",
    "Social",
    "Community",
    "Healthcare",
    "Household",
    "Extended Family",
    "Social",
    "Community",

    
    "Duration of Exposure (day)",
]

y = []
x = []
x_lower = []
x_upper = []
colors = []
edgecolors = []

list_text = []
list_text_full = []
x_text =[]
y_text = []
text_colors = []

for i, predictor in enumerate(list_predictors):
    y.append(len(list_predictors) - i)
    idx = dic_predictors_idx[predictor]
    if idx != "na":
        x.append(odds_ratios_rna.loc[idx, "Estimate"])
        x_lower.append(odds_ratios_rna.loc[idx, "lower"])
        x_upper.append(odds_ratios_rna.loc[idx, "upper"])
        x_text.append(odds_ratios_rna.loc[idx, "Estimate"])
        if odds_ratios_rna.loc[idx, "Pr...z.."] < 0.001:
            significance = "***"
        elif odds_ratios_rna.loc[idx, "Pr...z.."] < 0.01:
            significance = "**"
        elif odds_ratios_rna.loc[idx, "Pr...z.."] < 0.05:
            significance = "*"
        elif odds_ratios_rna.loc[idx, "Pr...z.."] < 0.1:
            significance = ""# (p="+str(round(odds_ratios_imputed_v1.loc[idx, "p.value"], 3))+")"
        else:
            significance = ""
        list_text.append(str(round(odds_ratios_rna.loc[idx, "Estimate"], 2))+significance)
        list_text_full.append(str(round(odds_ratios_rna.loc[idx, "Estimate"], 2))+significance+" ("+str(round(odds_ratios_rna.loc[idx, "lower"], 2))+", "+str(round(odds_ratios_rna.loc[idx, "upper"], 2))+")")
        y_text.append(len(list_predictors) - i)
    else:
        x.append(1)
        x_lower.append(1)
        x_upper.append(1)
        x_text.append(1)
        list_text.append("  Ref.")
        list_text_full.append(" Ref.")
        y_text.append(len(list_predictors) - i)
        
    if x_upper[i] < 1:
        colors.append(cm_2(0.7))
        edgecolors.append(cm_2(0.7))
        text_colors.append(cm_2(0.7))
    elif x_lower[i] > 1:
        colors.append(cm_1(0.7))
        edgecolors.append(cm_1(0.7))
        text_colors.append(cm_1(0.7))
    elif idx == "na":
        edgecolors.append("0")
        colors.append("1")
        text_colors.append("0.")
    else:
        colors.append("0.7")
        edgecolors.append("0.7")
        text_colors.append("0.")

ax.set_yticks(y)
ax.set_yticklabels(list_predictors_labels, fontsize=10)
ax.scatter(x, y,  marker="o", c=colors, edgecolors=edgecolors, s=30, zorder=3)
ax.hlines(y, x_lower, x_upper, color=edgecolors, linewidth=0.5, zorder=10)
ax.vlines(1, 0.3, max(y)+0.5, linestyle="-", color=cm_2(0.7), zorder=1, linewidth=0.5)
ax.set_ylim([0.3, max(y)+0.5])

ax.set_xlabel("Odds Ratio", fontsize=10)
ax.fill_between([0.00, 100], [.3, .3], [1.4, 1.4], color=cm_2(0.05), zorder=0)
ax.fill_between([0.00, 100], [1.4, 1.4], [10.4, 10.4], color=cm_2(0.15), zorder=0)
ax.fill_between([0.00, 100], [10.4, 10.4], [12.4, 12.4], color=cm_2(0.05), zorder=0)
ax.fill_between([0.00, 100], [12.4, 12.4], [14.4, 14.4], color=cm_2(0.15), zorder=0)
ax.fill_between([0.00, 100], [14.4, 14.4], [16.4, 16.4], color=cm_2(0.05), zorder=0)
ax.fill_between([0.00, 100], [16.4, 16.4], [21.4, 21.4], color=cm_2(0.15), zorder=0)
ax.fill_between([0.00, 100], [21.4, 21.4], [23.4, 23.4], color=cm_2(0.05), zorder=0)
ax.fill_between([0.00, 100], [23.4, 23.4], [27.4, 27.4], color=cm_2(0.15), zorder=0)
ax.fill_between([0.00, 100], [27.4, 27.4], [29.4, 29.4], color=cm_2(0.05), zorder=0)
ax.fill_between([0.00, 100], [29.4, 29.4], [33.4, 33.4], color=cm_2(0.15), zorder=0)
color=cm_2(0.8)
ax.text(0.0012, 33.3, "Age (contact)\nsusceptibility", horizontalalignment="left", verticalalignment="top", color=color, fontsize=10)
ax.text(0.0012, 29.3, "Sex (contact)\nsusceptibility", horizontalalignment="left", verticalalignment="top", color=color, fontsize=10)
ax.text(0.0012, 27.3, "Age (case)\ninfectiousness", horizontalalignment="left", verticalalignment="top", color=color, fontsize=10)
ax.text(0.0012, 23.3, "Sex (case)\ninfectiousness", horizontalalignment="left", verticalalignment="top", color=color, fontsize=10)
ax.text(0.0012, 21.3, "Clinical severity (case)", horizontalalignment="left", verticalalignment="top", color=color, fontsize=10)
ax.text(0.0012, 16.3, "\"Fever\" (case)", horizontalalignment="left", verticalalignment="top", color=color, fontsize=10)
ax.text(0.0012, 14.3, "\"Dry cough\" (case)", horizontalalignment="left", verticalalignment="top", color=color, fontsize=10)
ax.text(0.0012, 12.3, "Travel history Wuhan (case)", horizontalalignment="left", verticalalignment="top", color=color, fontsize=10)
ax.text(0.0012, 10.3, "Contact settings", horizontalalignment="left", verticalalignment="top", color=color, fontsize=10, zorder=11)
ax.text(90, 5.5, "Pre  Jan. 25, 2020", horizontalalignment="right", verticalalignment="bottom", color="0", fontsize=8, zorder=11)
ax.text(0.0012, 5.3, "Post  Jan. 25, 2020", horizontalalignment="left", verticalalignment="top", color="0", fontsize=8, zorder=11)
ax.text(0.0012, 1.3, "Duration of exposure", horizontalalignment="left", verticalalignment="top", color=color, fontsize=10)
ax.hlines(5.4, 0.00, 100, linewidth=0.5, linestyle="--")
baseline_odds_str = str(round(odds_ratios_rna.loc["(Intercept)", "Estimate"], 2)) + " (" + str(round(odds_ratios_rna.loc["(Intercept)", "lower"], 2)) + ", "+str(round(odds_ratios_rna.loc["(Intercept)", "upper"], 2))+")"
ax.set_title("Baseline Odds: " + baseline_odds_str, loc="right", fontsize=10)
ax.spines['right'].set_visible(False)
ax.spines['top'].set_visible(False)
ax.set_xscale("log")
ax.set_xlim([0.001, 100])
ax.tick_params(axis='both', which='both', labelsize=10)
y_text = np.array(y_text) - 0.2
for x_t, y_t, text, color in zip(x_text, y_text, list_text, text_colors):
    ax.text(x_t, y_t, text, color=color, horizontalalignment="left", verticalalignment="top", fontsize=8)

plt.tight_layout()
plt.savefig("../plots/FigS4.pdf")