################################
#### Code to generate FigS5 ####
################################

#### Load required packages
import pandas as pd
import numpy as np
import cmocean
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
import matplotlib.gridspec as gridspec
from matplotlib import rc
import matplotlib as mpl
import pickle

cm_1 = cmocean.cm.amp
cm_2 = cmocean.cm.ice_r
cm_3 = cmocean.cm.tempo
cm_4 = cmocean.cm.rain
cm_5 = cmocean.cm.gray_r
cm_6 = cmocean.cm.matter

cm1 = plt.get_cmap("Reds")
cm2 = plt.get_cmap("Blues")
cm3 = plt.get_cmap("Greens")
cm4 = plt.get_cmap("Oranges")

#### Load data
with open("../data/time_series_transmission_by_contact_types.json", "rb") as rf:
    dic_time_series_transmission_by_contact_types = pickle.load(rf)
#### Create figure instance and plot
fig, ax = plt.subplots(figsize=(6, 3))

ax.plot(dic_time_series_transmission_by_contact_types["t"], dic_time_series_transmission_by_contact_types["household"], color=cm_1(0.6), marker="o", markersize=4, linewidth=1, linestyle="-", label="Household", zorder=2)
ax.plot(dic_time_series_transmission_by_contact_types["t"], dic_time_series_transmission_by_contact_types["family"], color=cm_2(0.6), marker="o", markersize=4, linewidth=1, linestyle = "--", label="Family", zorder=3)
ax.plot(dic_time_series_transmission_by_contact_types["t"], dic_time_series_transmission_by_contact_types["social"], color=cm_3(0.6), marker="o", markersize=4, linewidth=1, linestyle=":", label="Social", zorder=4)
ax.plot(dic_time_series_transmission_by_contact_types["t"], dic_time_series_transmission_by_contact_types["community"], color=cm4(0.6), marker="s", markersize=3.5,linewidth=1, linestyle="-", label="Community", zorder=5)
ax.plot(dic_time_series_transmission_by_contact_types["t"], dic_time_series_transmission_by_contact_types["healthcare"], color=cm_5(0.7), marker="s", markersize=3.5,linewidth=1, linestyle=":", label="Healthcare", zorder=7)

ax.legend(frameon=False, fontsize=8, ncol=3, markerscale=1.3, loc="upper left")
ax.set_ylim([-0.02, 0.82])
ax.tick_params(labelsize=8)
ax.set_ylabel("Relative contribution", fontsize=10)
ax.set_xticks([9, 16, 23, 30, 37, 43])
ax.set_xticklabels(["Jan-10", "Jan-17", "Jan-24", "Jan-31", "Feb-7", "Feb-14"], fontsize=8)

ax_twinx = ax.twinx()
with open("../data/incidence_time_series.json", "rb") as rf:
    incidence_time_series = pickle.load(rf)

ax_twinx.bar(incidence_time_series[0], incidence_time_series[1], color="0.85", width=1.02, zorder=1)
ax_twinx.set_xlim(8, 45)
ax_twinx.set_ylabel("Incidence",fontsize=10, rotation=-90, labelpad=13)
ax.set_zorder(ax_twinx.get_zorder()+1)
ax.set_facecolor((0, 0, 0, 0))
plt.tight_layout()
plt.savefig("../plots/FigS5.pdf")