################################
#### Code to generate FigS6 ####
################################

#### Load required packages
import pandas as pd
import numpy as np
import cmocean
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
import matplotlib.gridspec as gridspec
from matplotlib import rc
import matplotlib as mpl
import pickle

cm_1 = cmocean.cm.amp
cm_2 = cmocean.cm.ice_r
cm_3 = cmocean.cm.tempo
cm_4 = cmocean.cm.rain
cm_5 = cmocean.cm.gray_r
cm_6 = cmocean.cm.matter

#### Create figure instance
fig, ax = plt.subplots(figsize=[6, 3])

#### Load data
with open("../data/incidence_time_series_by_case_ascertainment.json", "rb") as rf:
    incidence_time_series_by_case_ascertainment = pickle.load(rf)

t = incidence_time_series_by_case_ascertainment[0]
contact_tracing = incidence_time_series_by_case_ascertainment[1]
passive_surveillance = incidence_time_series_by_case_ascertainment[1]

#### Plot
ax.bar(t, contact_tracing, label="contact tracing", color=cm_1(0.5))
ax.bar(t, passive_surveillance, bottom=contact_tracing, label="passive surveillance", color=cm_2(0.5))
ax.legend(frameon=False, fontsize=8, loc="upper left")
ax.set_ylabel("Incidence")
ax.set_xlabel("Symptom onset")

ax.set_xticks([0, 10, 20, 31, 41, 51, 60])
ax.set_xticklabels(["Jan-1st", "Jan-10th", "Jan-20th", "Feb-1st", "Feb-10th", "Feb-20th", "Mar-1st"], fontsize=8)
ax.set_xlim([0, 60])
ax.set_ylim([0, 100])
ax.vlines([26, 34], 0, 100, linestyle="--", linewidth=1, color="0.3")
ax.text(23, 90, "I", fontsize=12)
ax.text(29.5, 90, "II", fontsize=12)
ax.text(36, 90, "III", fontsize=12)

plt.savefig("../plots/FigS6.pdf")