################################
#### Code to generate FigS8 ####
################################

#### Load required packages
import numpy as np
from scipy import stats
import random
import cmocean
import matplotlib.pyplot as plt
import matplotlib.gridspec as gridspec
from matplotlib import rc
import matplotlib as mpl
from matplotlib.colorbar import Colorbar
from copy import deepcopy
import pickle

cm1 = cmocean.cm.amp
cm2 = cmocean.cm.ice_r
cm3 = cmocean.cm.tempo
cm4 = cmocean.cm.rain

#### Create figure instance
fig = plt.figure(figsize=(12, 6))
gs = gridspec.GridSpec(2, 3)
axes00 = fig.add_subplot(gs[0, 0])
axes01 = fig.add_subplot(gs[0, 1])
axes02 = fig.add_subplot(gs[0, 2])
axes10 = fig.add_subplot(gs[1, 0])
axes11 = fig.add_subplot(gs[1, 1])
axes12 = fig.add_subplot(gs[1, 2])
axes = [[axes00, axes01, axes02], [axes10, axes11, axes12]]

t_bounds = [(-1000, 26), (26, 34), (34, 100)]
titles = [["(A)", "(B)", "(C)"], ["(D)", "(E)", "(F)"]]
texts = [["Phase I", "Phase II", "Phase III"], ["Phase I", "Phase II", "Phase III"]]
file_names = ["../data/array_onset_to_iso_phase_I.npy", "../data/array_onset_to_iso_phase_II.npy", "../data/array_onset_to_iso_phase_III.npy"]
list_params_fit_onset_to_iso = []

for idx, (t0, t1) in enumerate(t_bounds):
    temp_array = np.load(file_names[idx])
    temp_list = temp_array.flatten()

    median_str = str((round(np.median(temp_list), 1)))
    lower_str = str((round(np.percentile(temp_list, 25), 1)))
    upper_str = str((round(np.percentile(temp_list, 75), 1)))

    bin_width = (max(temp_list) - min(temp_list))/16.
    axes[1][idx].hist(
        temp_list,
        bins=15,
        color="0.8",
        density=True,
        width=bin_width,
        label="data")

    x_fit = [i*0.01 for i in range(int(min(temp_list)//0.01), int(max(temp_list)/0.01))]

    #### fit normal distribution ####
    loc, scale = stats.norm.fit(temp_list)
    y_fit_norm = stats.norm.cdf(x_fit, loc, scale)
    y_fit_norm_pdf = stats.norm.pdf(x_fit, loc, scale)
    AIC = 2 - int(stats.norm.logpdf(temp_list, loc, scale).sum())
    if idx == 1:
        list_params_fit_onset_to_iso.append({"name": "norm", "loc": loc, "scale": scale})
        axes[0][idx].plot(
            x_fit, y_fit_norm,
            color=cm1(0.7),
            linewidth=2.,
            linestyle="-",
            label="normal: AIC="+str(AIC))
        axes[1][idx].plot(
            x_fit, y_fit_norm_pdf,
            color=cm1(0.7),
            linewidth=2.,
            linestyle="-",
            label="best fit")


    #### fit gamma distribution ####
    shape, loc, scale = stats.gamma.fit(temp_list)
    y_fit_gamma = stats.gamma.cdf(x_fit, shape, loc, scale=scale)
    AIC = 3 - int(stats.gamma.logpdf(temp_list, shape, loc, scale).sum())
    if idx == 0:
        axes[0][idx].plot(
            x_fit, y_fit_gamma,
            color=cm3(0.7),
            linewidth=1.,
            linestyle=":",
            label="gamma: AIC="+str(AIC))
    if idx == 1:
        axes[0][idx].plot(
            x_fit, y_fit_gamma,
            color=cm3(0.7),
            linewidth=1.,
            linestyle="--",
            label="gamma: AIC="+str(AIC))
        
    #### fit cauchy distribution ####
    loc, scale = stats.cauchy.fit(temp_list)
    y_fit_cauchy = stats.cauchy.cdf(x_fit, loc, scale=scale)
    AIC = 2 - int(stats.cauchy.logpdf(temp_list, loc, scale).sum())
    
    #### fit laplace distribution ####
    loc, scale = stats.laplace.fit(temp_list)
    y_fit_laplace = stats.laplace.cdf(x_fit, loc, scale=scale)
    y_fit_laplace_pdf = stats.laplace.pdf(x_fit, loc, scale=scale)
    AIC = 2 - int(stats.laplace.logpdf(temp_list, loc, scale).sum())

    if idx == 2:
        list_params_fit_onset_to_iso.append({"name": "laplace", "loc": loc, "scale": scale})
        axes[0][idx].plot(
            x_fit, y_fit_laplace,
            color=cm1(0.7),
            linewidth=2.,
            linestyle="-",
            label="laplace: AIC="+str(AIC))
        axes[1][idx].plot(
            x_fit, y_fit_laplace_pdf,
            color=cm1(0.7),
            linewidth=2.,
            linestyle="-",
            zorder=100,
            label="best fit")

    
    #### fit gen gamma distribution ####
    a, c, loc, scale = stats.gengamma.fit(temp_list)
    y_fit_gengamma = stats.gengamma.cdf(x_fit, a, c, loc, scale=scale)
    AIC = 4 - int(stats.gengamma.logpdf(temp_list, a, c, loc, scale).sum())
    if idx == 2:
        axes[0][idx].plot(
            x_fit, y_fit_gengamma,
            color=cm3(0.7),
            linewidth=1.,
            linestyle=":",
            label="gen. gamma: AIC="+str(AIC))
    if idx == 0:
        axes[0][idx].plot(
            x_fit, y_fit_gengamma,
            color=cm2(0.7),
            linewidth=1.,
            linestyle="--",
            label="gen. gamma: AIC="+str(AIC))
    
    #### fit lognormal distribution ####
    shape, loc, scale = stats.lognorm.fit(temp_list)
    y_fit_lognorm = stats.lognorm.cdf(x_fit, shape, loc, scale=scale)
    y_fit_lognorm_pdf = stats.lognorm.pdf(x_fit, shape, loc, scale=scale)
    AIC = 3 - int(stats.lognorm.logpdf(temp_list, shape, loc, scale).sum())
    if idx == 0:
        list_params_fit_onset_to_iso.append({"name": "lognorm", "shape": shape, "loc": loc, "scale": scale})
        axes[0][idx].plot(
            x_fit, y_fit_lognorm,
            color=cm1(0.7),
            linewidth=2.,
            linestyle="-",
            label="lognormal: AIC="+str(AIC))
        axes[1][idx].plot(
            x_fit, y_fit_lognorm_pdf,
            color=cm1(0.7),
            linewidth=2.,
            linestyle="-",
            zorder=100,
            label="best fit")    
    if idx == 1:
        axes[0][idx].plot(
            x_fit, y_fit_lognorm,
            color=cm2(0.7),
            linewidth=1.,
            linestyle=":",
            label="lognorm: AIC="+str(AIC))

    #### fit beta distribution ####
    a, b, loc, scale = stats.beta.fit(temp_list)
    y_fit_beta = stats.beta.cdf(x_fit, a, b, loc, scale=scale)
    AIC = 4 - int(stats.beta.logpdf(temp_list, a, b, loc, scale).sum())
    if idx == 2:
        axes[0][idx].plot(
            x_fit, y_fit_beta,
            color=cm2(0.7),
            linewidth=1.,
            linestyle=":",
            label="beta: AIC="+str(AIC))

    #### visualize uncertainty
    x_fit_CI = np.linspace(min(temp_list), max(temp_list),  15)
    array_y_fit_cdf = []
    for row in temp_array:
        #### plot cdf for each of the 100 realizations.
        row_sorted = np.sort(row)
        cdf = 1. * np.arange(len(row_sorted)) / (len(row_sorted) - 1)
        axes[0][idx].plot(row_sorted, cdf, linewidth=0.5, color=cm2(0.3), zorder=0)
        #### fit to each of the 100 realizations.
        if idx == 0:
            shape, loc, scale = stats.lognorm.fit(row)
            y_fit_lognorm = stats.lognorm.cdf(row, shape, loc, scale=scale)
            array_y_fit_cdf.append(stats.lognorm.cdf(x_fit_CI, shape, loc, scale=scale))
        elif idx == 1:
            loc, scale = stats.norm.fit(row)
            array_y_fit_cdf.append(stats.norm.cdf(x_fit_CI, loc, scale))
        else:
            loc, scale = stats.laplace.fit(row)
            array_y_fit_cdf.append(stats.laplace.cdf(x_fit_CI, loc, scale=scale))

    array_y_fit_cdf = np.array(array_y_fit_cdf)
    y_fit_cdf_lower = np.percentile(array_y_fit_cdf, 2.5, axis=0)
    y_fit_cdf_upper = np.percentile(array_y_fit_cdf, 97.5, axis=0)
    y_fit_cdf_avg = np.average(array_y_fit_cdf, axis=0)
    axes[0][idx].vlines(x_fit_CI, y_fit_cdf_lower, y_fit_cdf_upper, zorder=101, linewidth=1, color=cm1(0.7))


    #### labeling
    axes[0][idx].legend(fontsize=6, frameon=False, loc="lower right")
    axes[0][idx].tick_params(axis="both", labelsize=8)
    axes[0][idx].set_xlabel("Onset to isolation (days)", fontsize=8)
    axes[0][idx].set_ylabel("Cumulative distribution function", fontsize=8)
    axes[0][idx].set_title(titles[0][idx], fontsize=10, loc="left")
    axes[0][idx].text(0.05, 0.9, texts[0][idx], fontsize=10, transform=axes[0][idx].transAxes)

    axes[1][idx].legend(fontsize=6, frameon=False, loc="upper right")
    axes[1][idx].set_xlabel("Onset to isolation (days)", fontsize=8)
    axes[1][idx].set_ylabel("Probability density function", fontsize=8)
    axes[1][idx].set_title(titles[1][idx], fontsize=10, loc="left")
    axes[1][idx].text(0.05, 0.9, texts[1][idx], fontsize=10, transform=axes[1][idx].transAxes)

plt.tight_layout()
plt.savefig("../plots/FigS8.pdf")