#### Code for transmission chain reconstruction.
#### Note that the original data cannot be made public due to privacy restrictions.
#### The data loaded here is only a mocked version of the data that do not reflect the reality.
#### The following code could only run properly with the original data. (CODE WON'T RUN WITH MOCK DATA).
#### However the mocked data demonstrate the data structure of the orginal database, which the code relies on.
#### The output are comment out in the code, the output generated with the original data are provided in the "../data/sampled_chains/"" folder

#### Load packages 
import pandas as pd
import networkx as nx
import numpy as np
import math
from webweb import Web
import matplotlib.pyplot as plt
from datetime import datetime
from scipy import stats
import random
from copy import deepcopy
from networkx.readwrite import json_graph
import pystan
from os import path
import arviz
import pickle
import sys
import cmocean
import matplotlib.gridspec as gridspec
from matplotlib import rc
import matplotlib as mpl
from matplotlib.colorbar import Colorbar

cm_1 = cmocean.cm.amp
cm_2 = cmocean.cm.ice_r
cm_3 = cmocean.cm.tempo
cm_4 = cmocean.cm.rain
cm_5 = cmocean.cm.gray_r

from numeric_day_converter import to_num_date_rand
from numeric_day_converter import to_num_date_discrete

#### Load linelist data 
df_linelist = pd.read_excel("../data_raw/MOCK_df_linelist.xlsx")
df_linelist = df_linelist.set_index("id")


#### Pruning impossible "source_ids"
dT = 1
list_pruned_infector_infectee = []
df_linelist["source_ids_pruned"] = ["na" for i in range(len(df_linelist))]
for idx, row in df_linelist.iterrows():
    
    # Get infectee exposure window
    expo_min_infectee = row["expo_min"]
    expo_max_infectee = row["expo_max"]
    
    # Get source_ids 
    if row["source_ids"] == "na":
        if row["travel_Wuhan"] == "1":
            df_linelist.loc[idx, "source_ids_pruned"] = "wuhan"
        else:
            df_linelist.loc[idx, "source_ids_pruned"] = "na"
    else:
        source_ids = set(row["source_ids"].strip().split(","))
        source_ids.discard("unk")
        # Add Wuhan as one of the source
        source_ids_str = "wuhan," if row["travel_Wuhan"] == "1" else ""
        for source_id in source_ids:
            # Get infector's exposure window
            expo_min_infector = df_linelist.loc[source_id, "expo_min"]
            expo_max_infector = df_linelist.loc[source_id, "expo_max"]
            if expo_min_infector + dT < expo_max_infectee:
                source_ids_str += (source_id+",")
            else:
                list_pruned_infector_infectee.append((source_id, idx))

        if source_ids_str == "":
            df_linelist.loc[idx, "source_ids_pruned"] = "na"
        else:
            df_linelist.loc[idx, "source_ids_pruned"] = source_ids_str.strip(",")

#### update exposure constrains based on the transmission chain topology.
G_clusters_core = nx.DiGraph()
for idx, row in df_linelist.iterrows():
    
    # Get source_ids
    source_ids = row["source_ids_pruned"].strip().split(",")
    if len(source_ids) == 0:
        source_ids = "na"
            
    G_clusters_core.add_node(
        idx,
        name=idx+"-"+row["source_ids_pruned"],
        generation=row["generation"] if row["generation"] != "na" else "na",
        source_ids=source_ids,
        onset=to_num_date_discrete(row["onset_date"]) if row["onset_date"] != "na" else "na",
        iso=to_num_date_discrete(row["iso_date"]) if row["iso_date"] != "na" else "na",
        expo_min=row["expo_min"],
        expo_max=row["expo_max"],
        expo_Wuhan=True if row["travel_Wuhan"] == "1" else False,
        flag="na" # temporal
    )
df_linelist_cluster = df_linelist[df_linelist["cluster_id"] != "na"]
for cluster_id, df_cluster in df_linelist_cluster.groupby("cluster_id"):
    temp_G_cluster = nx.DiGraph()
    for idx, row in df_cluster.iterrows():
        if row["source_ids_pruned"] != "na" and row["generation"] in set(["1", "2", "3", "4"]):
            source_ids = set(row["source_ids_pruned"].strip().split(","))
            if "wuhan" not in source_ids:  #### Having "wuhan" as a potential exposure is a free pass of constrain relief
                for source_id in source_ids:
                    G_clusters_core.add_edge(source_id, idx)
                    temp_G_cluster.add_edge(source_id, idx)
    if not nx.is_directed_acyclic_graph(temp_G_cluster):
        print(idx, nx.is_directed_acyclic_graph(temp_G_cluster))
        
    dT = 1    
    # For each node, get all decendents and their lower bounds of exposure ("expo_max"), the minimum of all determines the lower bound of the node
    dic_node_expo_max = {}
    for node in temp_G_cluster.nodes:
        descendants = nx.descendants(temp_G_cluster, node)
        if len(descendants) > 0:
            expo_max_cand = []
            ls = []
            for d in descendants:
                l = nx.shortest_path_length(temp_G_cluster, node, d)
                ls.append(l)
                expo_max_cand.append(df_linelist.loc[d, "expo_max"] - l * dT)
            expo_min = df_linelist.loc[node, "expo_min"]
            expo_max = df_linelist.loc[node, "expo_max"]
            expo_max_update = min(min(expo_max_cand), expo_max)
            dic_node_expo_max[node] = expo_max_update

            
            # Check if any rule is broken (None)
            if expo_max_update < expo_min:
                print("node:", node, "expo_min", expo_min, "expo_max", expo_max, "expo_max_update", expo_max_update, "expo_max_cand", expo_max_cand, ls, descendants)
            
    for node in dic_node_expo_max:
        df_linelist.loc[node, "expo_max"] = dic_node_expo_max[node]

#### Additional exposures for nodes in generation "1", "-1", and "na"
df_linelist["source_ids_secondary"] = ["na" for i in range(len(df_linelist))]
df_linelist_cluster = df_linelist[df_linelist["cluster_id"] != "na"]
for cluster_id, df_cluster in df_linelist_cluster.groupby("cluster_id"):
    # Generation 1
    df_gen1 = df_cluster[df_cluster["generation"] == "1"]
    if len(df_gen1) > 1:
        for infectee in df_gen1.index:
            source_ids_str = ""
            for infector in df_gen1.index:
                if infectee != infector:
                    source_ids_str += (infector+",")
            if source_ids_str == "":
                source_ids_str = "na"
            source_ids_str = source_ids_str.strip(",")
            df_linelist.loc[infectee, "source_ids_secondary"] = source_ids_str
    # Generation "na" and "-1"
    for idx, row in df_cluster.iterrows():
        if row["generation"] in set(["na", "-1"]):
            source_ids = set(df_cluster.index)
            source_ids.discard(idx)
            source_ids_str = ""
            for source_id in source_ids:
                source_ids_str += (source_id+",")
            if source_ids_str == "":
                source_ids_str = "na"
            source_ids_str = source_ids_str.strip(",")
            df_linelist.loc[idx, "source_ids_secondary"] = source_ids_str

#### Sample the transmission chains
for i in range(0, 100):
    df_linelist["onset_est"] = ["na" for i in range(len(df_linelist))]
    df_linelist["expo_est"] = ["na" for i in range(len(df_linelist))]
    df_linelist["source_est"] = ["na" for i in range(len(df_linelist))]
    df_linelist["iso_est"] = ["na" for i in range(len(df_linelist))]
    for idx, row in df_linelist.iterrows():
        onset_est = to_num_date_rand(row["onset_date"]) if row["onset_date"] != "na" else "na"
        iso_est = to_num_date_rand(row["iso_date"]) if row["iso_date"] != "na" else "na"
        df_linelist.loc[idx, "onset_est"] = onset_est
        df_linelist.loc[idx, "iso_est"] = iso_est

    #### Estimate exposure time.
    for cluster_id, df_cluster in df_linelist.groupby("cluster_id"):

        # generation 1, "na", "-1" (this also include cases don't belong to any cluster)
        for idx, row in df_cluster[df_cluster["generation"].isin(["1", "na", "-1"])].iterrows():
            onset_est = row["onset_est"]
            expo_min = row["expo_min"]
            expo_max = row["expo_max"]
            if onset_est != "na":
                flag = 0
                while(flag == 0):
                    incubation = stats.weibull_min.rvs(1.578104 , scale=7.112123)
                    expo_est = onset_est - incubation
                    if expo_est >= expo_min and expo_est <= expo_max:
                        df_linelist.loc[idx, "expo_est"] = expo_est
                        flag = 1        
            elif expo_max - expo_min < 3:
                expo_est = np.random.uniform(expo_min, expo_max)
                df_linelist.loc[idx, "expo_est"] = expo_est
            else:
                expo_est = "na"
                df_linelist.loc[idx, "expo_est"] = expo_est

        # generation 2-4 (upper-bound of exposure is also confined by exposure time of the source)
        for gen in ["2", "3", "4"]:
            for idx, row in df_cluster[df_cluster["generation"] == gen].iterrows():
                onset_est = row["onset_est"]
                list_expo_min = [row["expo_min"]]
                #### upper-bound of exposure is also confined by exposure time of the source
                for source_id in row["source_ids_pruned"].split(","):
                    if source_id not in set(["wuhan", "na"]) and df_linelist.loc[source_id, "expo_est"] != "na":
                        list_expo_min.append(df_linelist.loc[source_id, "expo_est"])
                expo_min = max(list_expo_min) 
                expo_max = row["expo_max"]
                if expo_min >= expo_max:
                    print("impossible exposure window", idx, expo_min, expo_max, list_expo_min)
                if onset_est != "na":
                    flag = 0
                    while(flag == 0):
                        incubation = stats.weibull_min.rvs(1.578104, scale=7.112123)
                        expo_est = onset_est - incubation
                        if expo_est >= expo_min and expo_est <= expo_max:
                            df_linelist.loc[idx, "expo_est"] = expo_est
                            flag = 1        
                elif expo_max - expo_min < 3:
                    expo_est = np.random.uniform(expo_min, expo_max)
                    df_linelist.loc[idx, "expo_est"] = expo_est
                else:
                    expo_est = "na" 
                    df_linelist.loc[idx, "expo_est"] = expo_est

    #### Select source of exposure among plausible candidate.
    for idx, row in df_linelist.iterrows():
        generation = row["generation"]
        expo_est = row["expo_est"]
        onset_set = row["onset_est"]

        # Get generation 1 exposure
        if generation == "1":
            set_source_ids = set(row["source_ids_pruned"].split(",") + row["source_ids_secondary"].split(","))
            set_source_ids.discard("na")



            source_ids_selected = []
            for source in set_source_ids:
                if source == "wuhan":
                    source_ids_selected.append(source)
                elif expo_est != "na" and df_linelist.loc[source, "expo_est"] != "na" and df_linelist.loc[source, "expo_est"] < expo_est:
                    source_ids_selected.append(source)
                else:
                    pass
            if source_ids_selected == []:
                source_ids_selected.append("community")
            source_est = random.choice(source_ids_selected)
            # checking for loop
            if source_est in df_linelist.index:
                if df_linelist.loc[source_est, "source_est"] != idx:
                    df_linelist.loc[idx, "source_est"] = source_est
                else:
                    df_linelist.loc[idx, "source_est"] = "community"
            else:
                df_linelist.loc[idx, "source_est"] = source_est

        # Get generation 2-4 exposure
        elif generation in set(["2", "3", "4"]):
            set_source_ids = set(row["source_ids_pruned"].split(","))
            source_ids_selected = []
            for source in set_source_ids:
                if source == "wuhan":
                    source_ids_selected.append(source)
                elif source == "na":
                    source_ids_selected.append("community")
                elif expo_est != "na" and df_linelist.loc[source, "expo_est"] != "na":
                    if df_linelist.loc[source, "expo_est"] < expo_est:
                        source_ids_selected.append(source)
                    else:
                        print("problematic", "source:", source, "idx:", idx, "source_expo:", df_linelist.loc[source, "expo_est"], "idx_expo:", expo_est)
                else:
                    source_ids_selected.append(source)
            source_est = random.choice(source_ids_selected)

            # checking for loop
            if source_est in df_linelist.index:
                if df_linelist.loc[source_est, "source_est"] != idx:
                    df_linelist.loc[idx, "source_est"] = source_est
                else:
                    #df_linelist.loc[idx, "source_est"] = "community"
                    print("trouble, loop")
            else:
                df_linelist.loc[idx, "source_est"] = source_est

        # Get generation "na" exposure
        elif generation == "na":
            set_source_ids = set(row["source_ids_secondary"].strip().split(","))
            if row["travel_Wuhan"] == "1":
                set_source_ids.add("wuhan")
            set_source_ids.discard("na")
            source_ids_selected = []
            for source in set_source_ids:
                if source == "wuhan":
                    source_ids_selected.append(source)
                elif expo_est != "na" and df_linelist.loc[source, "expo_est"] != "na" and df_linelist.loc[source, "expo_est"] < expo_est:
                    source_ids_selected.append(source)
                else:
                    pass
            if source_ids_selected == []:
                source_ids_selected.append("community")
            source_est = random.choice(source_ids_selected)

            # checking for loop
            if source_est in df_linelist.index:
                if df_linelist.loc[source_est, "source_est"] != idx:
                    df_linelist.loc[idx, "source_est"] = source_est
                else:
                    df_linelist.loc[idx, "source_est"] = "community"
            else:
                df_linelist.loc[idx, "source_est"] = source_est

        # Get generation "-1" exposure
        elif generation == "-1":
            set_source_ids = set(row["source_ids_pruned"].strip().split(","))
            if row["travel_Wuhan"] == "1":
                set_source_ids.add("wuhan")
            set_source_ids.discard("na")
            if len(set_source_ids) == 0:
                print("trouble")
            else:
                source_ids_selected = []
                for source in set_source_ids:
                    if source == "wuhan":
                        source_ids_selected.append(source)
                    elif expo_est != "na" and df_linelist.loc[source, "expo_est"] != "na" and df_linelist.loc[source, "expo_est"] < expo_est:
                        source_ids_selected.append(source)
                    else:
                        pass
                if source_ids_selected != []:
                    source_est = random.choice(source_ids_selected)

                    #### checking for loop
                    if source_est in df_linelist.index:
                        if df_linelist.loc[source_est, "source_est"] != idx:
                            df_linelist.loc[idx, "source_est"] = source_est
                        else:
                            df_linelist.loc[idx, "source_est"] = "community"
                    else: 
                        df_linelist.loc[idx, "source_est"] = source_est


                else:
                    set_source_ids = set(row["source_ids_secondary"].strip().split(","))
                    set_source_ids.discard("na")
                    source_ids_selected = []
                    for source in set_source_ids:
                        if expo_est != "na" and df_linelist.loc[source, "expo_est"] != "na" and df_linelist.loc[source, "expo_est"] < expo_est:
                            source_ids_selected.append(source)
                        else:
                            pass#source_ids_selected.append(source) !!!!
                    if source_ids_selected == []:
                        source_ids_selected.append("community")
                    source_est = random.choice(source_ids_selected)

                    #### checking for loop
                    if source_est in df_linelist.index:
                        if df_linelist.loc[source_est, "source_est"] != idx:
                            df_linelist.loc[idx, "source_est"] = source_est
                        else:
                            df_linelist.loc[idx, "source_est"] = "community"
                    else:
                        df_linelist.loc[idx, "source_est"] = source_est

    #### Create transmission chains
    dic_top_cities = {
        "Changsha": "1 Changsha",
        "Yueyang": "2 Yueyang",
        "Shaoyang": "3 Shaoyang",
        "Loudi": "4 Loudi",
        "Changde": "5 Changde",
        "Zhuzhou": "6 Zhuzhou",
        "Yiyang": "7 Yiyang"}


    G_chains = nx.DiGraph()
    for idx, row in df_linelist.iterrows():

        # Add nodes
        if row["expo_est"] != "na":
            name = idx+"-"+str(round(row["expo_est"],2))
        else:
            name = idx+"-na"
        G_chains.add_node(
            idx,
            name = name,
            generation = "na",
            generation_raw = row["generation"],
            date_onset = row["onset_est"],
            date_onset_str = row["onset_date"],
            date_iso = row["iso_est"],
            expo_est = row["expo_est"],
            date_diag = to_num_date_rand(row["diag_date"]) if row["diag_date"] != "na" else "na",
            age = row["age"],
            gender = "male" if row["sex"] == "1" else "female",
            clinic_type = row["clinical_type"],
            diag_type = row["diag_type"], 
            loc = row["reporting_city"],
            loc_viz=dic_top_cities[row["reporting_city"]] if row["reporting_city"] in dic_top_cities else "Others",
            cluster_id = row["cluster_id"],
            cluster_type = "na",
            symp_upper = row["symptoms_upper_tract"],
            R0 = "na"    
        )

        # Add edges
        if row["source_est"] in df_linelist.index:
            G_chains.add_edge(row["source_est"], idx)
        clusters_chains = nx.weakly_connected_components(G_chains)

        # Calculate generation and degree
        nodes = G_chains.nodes(data=True)
        for cluster_chains in clusters_chains:
            G_cluster_chains = G_chains.subgraph(cluster_chains)
            root_node = [i[0] for i in G_cluster_chains.in_degree() if i[1] == 0][0]
            for node in G_cluster_chains:
                nodes[node]["generation"] = "g"+ str(nx.shortest_path_length(G_cluster_chains, root_node, node))
                nodes[node]["R0"] = G_cluster_chains.out_degree(node)

    #### We do not save the chains here since it's created on the mock data,
    #### In the "../data/sampled_chains/" however 100 realizations of reconstructed transmission chains based on the original data with only topolgy and location information (all other data attribute removed) are available.
    #nx.write_gpickle(G_chains, "../data/sampled_chains/MOCK_G_chains_"+str(i)+ ".gpickle")
    #print(i)