

library(tidyverse)
library(MASS)

df_regression <- read_csv("../data_raw/MOCK_df_regression_contacts_R.csv",
                          col_types=cols(
                            infector_id="c",
                            community_contacts="d",
                            social_contacts="d",
                            family_contacts="d",
                            household_contacts="d",
                            community_degree="d",
                            social_degree="d",
                            family_degree="d",
                            household_degree="d",
                            age_cat="f",
                            sex="f",
                            symp_fever="f",
                            symp_dry_cough="f",
                            severity="f",
                            travel_Wuhan="f",
                            intervention_phase="f",
                            neg_avg_mobility_index="d",
                            pre_01_25="f",
                            iso_to_onset="d"
                          ),
                          na="na")

df_regression$age_cat = relevel(df_regression$age_cat, ref="26-64")
df_regression$sex = relevel(df_regression$sex, ref="female")
df_regression$symp_fever = relevel(df_regression$symp_fever, ref="0")
df_regression$symp_dry_cough = relevel(df_regression$symp_dry_cough, ref="0")
df_regression$severity = relevel(df_regression$severity, ref="2")
df_regression$travel_Wuhan = relevel(df_regression$travel_Wuhan, ref="0")
df_regression$pre_01_25 = relevel(df_regression$pre_01_25, ref="1")


#### 1. regression community_cotacts
m <- glm.nb(community_contacts ~  age_cat + sex + symp_fever + symp_dry_cough + travel_Wuhan + pre_01_25 + iso_to_onset, data=df_regression, link="log")
coefs <- summary(m)$coefficients
coefs <- data.frame(coefs)
options(scipen = 50)
coefs["lower"] <- coefs["Estimate"] - 1.96*coefs["Std..Error"]
coefs["upper"] <- coefs["Estimate"] + 1.96*coefs["Std..Error"]

odds_ratios_community_contacts = exp(coefs[, c("Estimate", "lower", "upper", "Pr...z..")])
odds_ratios_community_contacts["Pr...z.."] = log(odds_ratios_community_contacts["Pr...z.."])
#write.csv(x=odds_ratios_community_contacts, file="../data/contact_pattern_odds_ratios_community.csv")
theta1=m$theta


#### 2. regression social_contacts
m <- glm.nb(social_contacts ~ age_cat + sex + symp_fever + symp_dry_cough + travel_Wuhan + pre_01_25 + iso_to_onset, data=df_regression, link="log")
coefs <- summary(m)$coefficients
coefs <- data.frame(coefs)
options(scipen = 50)
coefs["lower"] <- coefs["Estimate"] - 1.96*coefs["Std..Error"]
coefs["upper"] <- coefs["Estimate"] + 1.96*coefs["Std..Error"]

odds_ratios_social_contacts = exp(coefs[, c("Estimate", "lower", "upper", "Pr...z..")])
odds_ratios_social_contacts["Pr...z.."] = log(odds_ratios_social_contacts["Pr...z.."])
#write.csv(x=odds_ratios_social_contacts, file="../data/contact_pattern_odds_ratios_social.csv")
theta2=m$theta


#### 3. regression family_contacts
m <- glm.nb(family_contacts ~ age_cat + sex + symp_fever + symp_dry_cough + travel_Wuhan + pre_01_25 + iso_to_onset, data=df_regression, link="log")
coefs <- summary(m)$coefficients
coefs <- data.frame(coefs)
options(scipen = 50)
coefs["lower"] <- coefs["Estimate"] - 1.96*coefs["Std..Error"]
coefs["upper"] <- coefs["Estimate"] + 1.96*coefs["Std..Error"]

odds_ratios_family_contacts = exp(coefs[, c("Estimate", "lower", "upper", "Pr...z..")])
odds_ratios_family_contacts["Pr...z.."] = log(odds_ratios_family_contacts["Pr...z.."])
#write.csv(x=odds_ratios_family_contacts, file="../data/contact_pattern_odds_ratios_family.csv")
theta3=m$theta


#### 4. regression household_contacts
m <- glm.nb(household_contacts ~ age_cat + sex  + symp_fever + symp_dry_cough + travel_Wuhan + pre_01_25 + iso_to_onset, data=df_regression, link="log")

coefs <- summary(m)$coefficients
coefs <- data.frame(coefs)
options(scipen = 50)
coefs["lower"] <- coefs["Estimate"] - 1.96*coefs["Std..Error"]
coefs["upper"] <- coefs["Estimate"] + 1.96*coefs["Std..Error"]

odds_ratios_household_contacts = exp(coefs[, c("Estimate", "lower", "upper", "Pr...z..")])
odds_ratios_household_contacts["Pr...z.."] = log(odds_ratios_household_contacts["Pr...z.."])
#write.csv(x=odds_ratios_household_contacts, file="../data/contact_pattern_odds_ratios_household.csv")
theta4=m$theta


list_thetas = c(theta1, theta2, theta3, theta4)
df_thetas = data.frame("theta"=list_thetas)
#write.csv(x=df_thetas, file="../data/contact_pattern_thetas.csv")
