#### Code of of regression model for Figure S3
#### Note that the original data cannot be made public due to privacy restrictions.
#### The data loaded here is only a mocked version of the data that do not reflect the reality.
#### The following code could only run properly with the original data. (CODE WON'T RUN WITH MOCK DATA).
#### However the mocked data demonstrate the data structure of the orginal database, which the code relies on.
#### The output are comment out in the code, the output generated with the original data are provided in the "../data" folder

library(lme4)
library(tidyverse)
library(broom.mixed)
library(mice)

########################################################################
#### logistic regression of transmission with missing value imputed ####
########################################################################


df_regression <- read_csv("../data/MOCK_df_regression_R_imp.csv",
                          col_types=cols(
                              index="c",
                              id="c",
                              infection_state="l",
                              age="d",
                              age_source="d",
                              age_cat="f",
                              age_cat_source="f",
                              sex="f",
                              sex_source="f",
                              contact_type="f",
                              intervention_phase_contact="f",
                              contact_type="f",
                              contact_type_updated_agg_v1="f",
                              contact_type_updated_agg_v2="f",
                              contact_type_updated_agg_v3="f",
                              severity_source="f",
                              severity_grouped_v1_source="f",
                              severity_grouped_v1_expo_source="f",
                              severity_grouped_v2_expo_source="f",
                              source_Wuhan="f",
                              symp_fever_source="f",
                              symp_dry_cough_source="f",
                              source_id_linelist="f",
                              onset_within_expo="f",
                              num_onset_discrete_source="d",
                              duration_of_expo="d",
                              duration_of_expo_cap="d",
                              expo_weight="d"
                          ),
                          na="na")


df_regression$age_cat <- relevel(as.factor(df_regression$age_cat), ref="26-64")
df_regression$age_cat_source <- relevel(as.factor(df_regression$age_cat_source), ref="26-64")
df_regression$sex <- relevel(as.factor(df_regression$sex), ref="female")
df_regression$sex_source <- relevel(as.factor(df_regression$sex_source), ref="female")
df_regression$source_Wuhan <- relevel(as.factor(df_regression$source_Wuhan), ref="0")
df_regression$intervention_phase_contact <- relevel(as.factor(df_regression$intervention_phase_contact), ref="I")
df_regression$contact_type <- relevel(as.factor(df_regression$contact_type), ref="Household contacts")
df_regression$contact_type_updated_agg_v1 <- relevel(as.factor(df_regression$contact_type_updated_agg_v1), ref="Family contacts")
df_regression$contact_type_updated_agg_v2 <- relevel(as.factor(df_regression$contact_type_updated_agg_v2), ref="Family contactsI")
df_regression$contact_type_updated_agg_v3 <- relevel(as.factor(df_regression$contact_type_updated_agg_v3), ref="Family contactsI")
df_regression$severity_source <- relevel(as.factor(df_regression$severity_source), ref="3")
df_regression$severity_grouped_v1_source <- relevel(as.factor(df_regression$severity_grouped_v1_source), ref="2")
df_regression$severity_grouped_v1_expo_source <- relevel(as.factor(df_regression$severity_grouped_v1_expo_source), ref="2-0")
df_regression$severity_grouped_v2_expo_source <- relevel(as.factor(df_regression$severity_grouped_v2_expo_source), ref="2-0")
df_regression$onset_within_expo <- relevel(as.factor(df_regression$onset_within_expo), ref="0")
df_regression$symp_fever_source <- relevel(as.factor(df_regression$symp_fever_source), ref="0")
df_regression$symp_dry_cough_source <- relevel(as.factor(df_regression$symp_dry_cough_source), ref="0")

#########################################
#### Multiple imputation using mice #####
#########################################

if (!file.exists("../data/R_df_regression_imputed.rds")){
    init = mice(df_regression, maxit=0) 
    meth = init$method
    predM = init$predictorMatrix
    predM[, c("index", "id", "index", "source_id_linelist", "expo_weight", "infection_state", "age_cat", "age_cat_source", "severity_grouped_v1_expo_source", "severity_grouped_v2_expo_source", "severity_grouped_v1_source", "onset_within_expo", "duration_of_expo_cap", "contact_type_updated_agg_v1", "contact_type_updated_agg_v2", "contact_type_updated_agg_v3")] = 0
    meth[c("id", "index", "source_id_linelist", "expo_weight", "infection_state")] = ""
    meth[c("age_cat", "age_cat_source", "contact_type_updated_agg_v1", "contact_type_updated_agg_v2", "contact_type_updated_agg_v3", "contact_type", "severity_grouped_v1_source", "severity_grouped_v1_expo_source", "severity_grouped_v2_expo_source", "contact_type")] = "polyreg"
    meth[c("sex", "source_Wuhan", "symp_fever_source", "symp_dry_cough_source", "onset_within_expo", "intervention_phase_contact")] = "logreg"
    meth[c("age", "duration_of_expo", "duration_of_expo_cap", "num_onset_discrete_source")] = "pmm"
    df_regression_imp <- mice(df_regression, maxit=10, m=10, method=meth, predictorMatrix=predM, printFlag=TRUE)
    saveRDS(df_regression_imp, file="../data/R_df_regression_imputed.rds")
}
#df_regression_imp <- readRDS("../data/R_df_regression_imputed.rds")


#################################
#### pooling fitting results ####
#################################

#### odds ratios
coefs <- df_regression_imp %>%
    with(glmer(infection_state ~ age_cat + age_cat_source + sex + sex_source + contact_type_updated_agg_v3 + severity_grouped_v1_source + symp_fever_source + symp_dry_cough_source + source_Wuhan + duration_of_expo_cap + onset_within_expo + (1|source_id_linelist), weights=expo_weight, nAGQ=0, family=binomial(link = "logit"))) %>%
    pool() %>%
    summary() %>%
    column_to_rownames(var="term")


    
coefs["lower"] <- coefs["estimate"] - 1.96*coefs["std.error"]
coefs["upper"] <- coefs["estimate"] + 1.96*coefs["std.error"]
coefs[, c("estimate", "lower", "upper")] <- exp(coefs[, c("estimate", "lower", "upper")])
odds_ratios_imputed <- coefs[, c("estimate", "lower", "upper", "p.value")]
#write.csv(x=odds_ratios_imputed, file="../data/transmission_odds_ratios_imputed.csv")

#### save imputed dataframe
df_regression_imputed <- complete(df_regression_imp, action="long", include=FALSE)
#write.csv(x=df_regression_imputed, file="../data/df_regression_imputed.csv")


#### AIC of contact type aggregation
df_regression_imputed$contact_type_merge0 <- df_regression_imputed$contact_type_updated_agg_v3 %>%
    fct_collapse(
        Ho=c("Household contactsI", "Household contactsII+"),
        Fa=c("Family contactsI", "Family contactsII+"),
        So=c("Social contactsI", "Social contactsII+"),
        Co=c("Community contactsI", "Community contactsII+"),
        He=c("Healthcare contactsI", "Healthcare contactsII+")
    )

df_regression_imputed$contact_type_merge1 <- df_regression_imputed$contact_type_merge0 %>%
    fct_collapse(
        HoFa=c("Ho", "Fa")
    )

df_regression_imputed$contact_type_merge2 <- df_regression_imputed$contact_type_merge1 %>%
    fct_collapse(
        HoFaSo=c("HoFa", "So")
    )

df_regression_imputed$contact_type_merge3 <- df_regression_imputed$contact_type_merge2 %>%
    fct_collapse(
        HoFaSoCo=c("HoFaSo", "Co")
    )
df_regression_imputed$contact_type_updated_agg_v3[1:10]
colnames(df_regression_imputed)


AICs_ref <- df_regression_imputed %>% 
    group_by(.imp) %>% 
    do(AICs=tidy(AIC(glmer(infection_state ~ age_cat + age_cat_source + sex + sex_source + contact_type_updated_agg_v3 + severity_grouped_v1_source + symp_fever_source + symp_dry_cough_source + source_Wuhan + duration_of_expo_cap + onset_within_expo + (1|source_id_linelist), data=., weights=expo_weight, nAGQ=0, family=binomial(link = "logit"))))) %>% 
    unnest(AICs) %>% pull(var=x)

AICs_merge0 <- df_regression_imputed %>% 
    group_by(.imp) %>% 
    do(AICs=tidy(AIC(glmer(infection_state ~ age_cat + age_cat_source + sex + sex_source + contact_type_merge0 + severity_grouped_v1_source + symp_fever_source + symp_dry_cough_source + source_Wuhan + duration_of_expo_cap + onset_within_expo + (1|source_id_linelist), data=., weights=expo_weight, nAGQ=0, family=binomial(link = "logit"))))) %>% 
    unnest(AICs) %>% pull(var=x)

AICs_merge1 <- df_regression_imputed %>% 
    group_by(.imp) %>% 
    do(AICs=tidy(AIC(glmer(infection_state ~ age_cat + age_cat_source + sex + sex_source + contact_type_merge1 + severity_grouped_v1_source + symp_fever_source + symp_dry_cough_source + source_Wuhan + duration_of_expo_cap + onset_within_expo + (1|source_id_linelist), data=., weights=expo_weight, nAGQ=0, family=binomial(link = "logit"))))) %>% 
    unnest(AICs) %>% pull(var=x)

AICs_merge2 <- df_regression_imputed %>% 
    group_by(.imp) %>% 
    do(AICs=tidy(AIC(glmer(infection_state ~ age_cat + age_cat_source + sex + sex_source + contact_type_merge2 + severity_grouped_v1_source + symp_fever_source + symp_dry_cough_source + source_Wuhan + duration_of_expo_cap + onset_within_expo + (1|source_id_linelist), data=., weights=expo_weight, nAGQ=0, family=binomial(link = "logit"))))) %>% 
    unnest(AICs) %>% pull(var=x)

AICs_merge3 <- df_regression_imputed %>% 
    group_by(.imp) %>% 
    do(AICs=tidy(AIC(glmer(infection_state ~ age_cat + age_cat_source + sex + sex_source + contact_type_merge3 + severity_grouped_v1_source + symp_fever_source + symp_dry_cough_source + source_Wuhan + duration_of_expo_cap + onset_within_expo + (1|source_id_linelist), data=., weights=expo_weight, nAGQ=0, family=binomial(link = "logit"))))) %>% 
    unnest(AICs) %>% pull(var=x)

AICs_merge4 <- df_regression_imputed %>% 
    group_by(.imp) %>% 
    do(AICs=tidy(AIC(glmer(infection_state ~ age_cat + age_cat_source + sex + sex_source + severity_grouped_v1_source + symp_fever_source + symp_dry_cough_source + source_Wuhan + duration_of_expo_cap + onset_within_expo + (1|source_id_linelist), data=., weights=expo_weight, nAGQ=0, family=binomial(link = "logit"))))) %>% 
    unnest(AICs) %>% pull(var=x)

df_AICs = data.frame(
    "M0"=mean(AICs_ref),
    "M1"=mean(AICs_merge0),
    "M2"=mean(AICs_merge1),
    "M3"=mean(AICs_merge2),
    "M4"=mean(AICs_merge3),
    "M5"=mean(AICs_merge4)
)
colMeans(df_AICs)
#write_csv(x=df_AICs, file="../data/AIC_score_regression_contact_cat_merge.csv")
