#### Code of of regression model for Figure S4
#### Note that the original data cannot be made public due to privacy restrictions.
#### The data loaded here is only a mocked version of the data that do not reflect the reality.
#### The following code could only run properly with the original data. (CODE WON'T RUN WITH MOCK DATA).
#### However the mocked data demonstrate the data structure of the orginal database, which the code relies on.
#### The output are comment out in the code, the output generated with the original data are provided in the "../data" folder

library(lme4)
library(tidyverse)


########################################################################
#### logistic regression of transmission with missing value removed ####
########################################################################

df_regression <- read_csv("../data/MOCK_df_regression_R_rna.csv",
                          col_types=cols(
                            index="c",
                            id="c",
                            infection_state="l",
                            age="d",
                            age_source="d",
                            age_cat="f",
                            age_cat_source_finer="f",
                            sex="f",
                            sex_source="f",
                            contact_type="f",
                            intervention_phase_contact="f",
                            contact_type="f",
                            contact_type_updated_agg_v1="f",
                            contact_type_updated_agg_v2="f",
                            contact_type_updated_agg_v3="f",
                            severity_source="f",
                            severity_grouped_v1_source="f",
                            severity_grouped_v1_expo_source="f",
                            severity_grouped_v2_expo_source="f",
                            source_Wuhan="f",
                            symp_fever_source="f",
                            symp_dry_cough_source="f",
                            source_id_linelist="f",
                            onset_within_expo="f",
                            num_onset_discrete_source="d",
                            duration_of_expo="d",
                            duration_of_expo_cap="d",
                            expo_weight="d"
                          ),
                          na="na")

df_regression$age_cat = relevel(as.factor(df_regression$age_cat), ref="26-64")
df_regression$age_cat_source_finer = relevel(as.factor(df_regression$age_cat_source_finer), ref="26-64")
df_regression$sex = relevel(as.factor(df_regression$sex), ref="female")
df_regression$sex_source = relevel(as.factor(df_regression$sex_source), ref="female")
df_regression$source_Wuhan = relevel(as.factor(df_regression$source_Wuhan), ref="0")
df_regression$intervention_phase_contact = relevel(as.factor(df_regression$intervention_phase_contact), ref="I")
df_regression$contact_type = relevel(as.factor(df_regression$contact_type), ref="Household contacts")
df_regression$contact_type_updated_agg_v1 = relevel(as.factor(df_regression$contact_type_updated_agg_v1), ref="Family contacts")
df_regression$contact_type_updated_agg_v2 = relevel(as.factor(df_regression$contact_type_updated_agg_v2), ref="Family contactsI")
df_regression$contact_type_updated_agg_v3 = relevel(as.factor(df_regression$contact_type_updated_agg_v3), ref="Family contactsI")
df_regression$severity_source = relevel(as.factor(df_regression$severity_source), ref="3")
df_regression$severity_grouped_v1_source = relevel(as.factor(df_regression$severity_grouped_v1_source), ref="2")
df_regression$severity_grouped_v1_expo_source = relevel(as.factor(df_regression$severity_grouped_v1_expo_source), ref="2-0")
df_regression$severity_grouped_v2_expo_source = relevel(as.factor(df_regression$severity_grouped_v2_expo_source), ref="2-0")
df_regression$onset_within_expo = relevel(as.factor(df_regression$onset_within_expo), ref="0")
df_regression$symp_fever_source = relevel(as.factor(df_regression$symp_fever_source), ref="0")
df_regression$symp_dry_cough_source = relevel(as.factor(df_regression$symp_dry_cough_source), ref="0")

m <- glmer(infection_state ~ age_cat + age_cat_source_finer + sex + sex_source + contact_type_updated_agg_v3 + severity_grouped_v1_expo_source + symp_fever_source + symp_dry_cough_source + source_Wuhan + duration_of_expo_cap + (1|source_id_linelist), data=df_regression, weights=expo_weight, nAGQ=0, family=binomial(link = "logit"))

coefs <- summary(m)$coefficients
coefs <- data.frame(coefs)
options(scipen = 50)
coefs["lower"] <- coefs["Estimate"] - 1.96*coefs["Std..Error"]
coefs["upper"] <- coefs["Estimate"] + 1.96*coefs["Std..Error"]

odds_ratios_rna = exp(coefs[, c("Estimate", "lower", "upper", "Pr...z..")])
odds_ratios_rna["Pr...z.."] = log(odds_ratios_rna["Pr...z.."])
#write.csv(x=odds_ratios_rna, file="../data/transmission_odds_ratios_rna.csv")
