#####################################################################
#### Code to generate data to produce phase diagram for Figure 2 ####
#####################################################################

#### Load required packages
import pandas as pd
import networkx as nx
import numpy as np
import math
from webweb import Web
import matplotlib.pyplot as plt
from datetime import datetime
from scipy import stats
from scipy import integrate
import random
from copy import deepcopy
from networkx.readwrite import json_graph
import pystan
import arviz
import matplotlib.ticker as ticker
from numba import cfunc
import scipy
import pickle
from os import path
import json
import cmocean
import matplotlib.gridspec as gridspec
from matplotlib import rc
import matplotlib as mpl
from matplotlib.colorbar import Colorbar

cm1 = cmocean.cm.amp
cm2 = cmocean.cm.ice_r
cm3 = cmocean.cm.tempo
cm4 = cmocean.cm.rain

list_GI_adj = np.load("../data/list_GI_adj.npy", allow_pickle=True)

def get_R0(shape_GI, scale_GI, shape_incu, scale_incu, avg_iso, std_iso, R0_Wuhan):
    
    def f(x0, x1, x2):
        a = shape_GI/scale_GI * (x2/scale_GI)**(shape_GI - 1)*np.exp(-(x2/scale_GI)**shape_GI)
        b = shape_incu/scale_incu * (x0/scale_incu)**(shape_incu - 1)*np.exp(-(x0/scale_incu)**shape_incu)
        c = 1/(std_iso*(2*np.pi)**.5)*np.exp(-0.5*((x1-x0-avg_iso)/std_iso)**2)
        return a*b*c
    
    def bounds_x0(x1, x2):
        return [0, np.inf]
    def bounds_x1(x2):
        return [-np.inf, x2]
    def bounds_x2():
        return [0, np.inf]
    
    return R0_Wuhan * (1-integrate.nquad(f, [bounds_x0, bounds_x1, bounds_x2], opts={"epsabs": 0.001, "limlst":10})[0])


if not path.isfile("../data/list_R0s_avg_iso.json"):
    growth_rate = 0.08
    shape, loc, scale = stats.weibull_min.fit(list_GI_adj, floc=0)
    def R_inverse(x, r, c, scale):
        ### generation interval
        GI = c/scale * (x/scale)**(c-1)*np.exp(-(x/scale)**c)
        return np.exp(-r*x) * GI
    R_inv, _ = integrate.quad(R_inverse, 0, np.inf, args=(growth_rate, shape, scale))
    R0_Wuhan = 1/R_inv
    
    list_avg_iso = np.linspace(-2, 2, 51)
    list_R0s = []
    for c0, avg_iso in enumerate(list_avg_iso):
        list_R0s.append(get_R0(shape, scale, 1.578104,  7.112123, avg_iso, 2, R0_Wuhan))
        print(c0, avg_iso, list_R0s[-1])
    with open("../data/list_R0s_avg_iso.json", "wb") as wf:
        pickle.dump([list_R0s, list_avg_iso], wf)
else:
    with open("../data/list_R0s_avg_iso.json", "rb") as rf:
        list_R0s, list_avg_iso = pickle.load(rf)


if not path.isfile("../data/list_R0s_upper_avg_iso.json"):
    growth_rate = 0.15
    shape, loc, scale = stats.weibull_min.fit(list_GI_adj, floc=0)
    def R_inverse(x, r, c, scale):
        ### generation interval
        GI = c/scale * (x/scale)**(c-1)*np.exp(-(x/scale)**c)
        return np.exp(-r*x) * GI
    R_inv, _ = integrate.quad(R_inverse, 0, np.inf, args=(growth_rate, shape, scale))
    R0_Wuhan = 1/R_inv
    
    list_avg_iso = np.linspace(-5, 0, 51)
    list_R0s = []
    for c0, avg_iso in enumerate(list_avg_iso):
        list_R0s.append(get_R0(shape, scale, 1.578104,  7.112123, avg_iso, 2, R0_Wuhan))
        print(c0, avg_iso, list_R0s[-1])
    with open("../data/list_R0s_upper_avg_iso.json", "wb") as wf:
        pickle.dump([list_R0s, list_avg_iso], wf)
else:
    with open("../data/list_R0s_upper_avg_iso.json", "rb") as rf:
        list_R0s_upper, list_avg_iso_upper = pickle.load(rf)